@@ -212,11 +212,13 @@ _theora_ilog (unsigned int v)
   return (ret);
 }
 
+/* Return the frame number (starting from zero) corresponding to this 
+ * granulepos */
 static gint64
 _theora_granule_frame (GstTheoraDec * dec, gint64 granulepos)
 {
   guint ilog;
-  gint framecount;
+  gint framenum;
 
   if (granulepos == -1)
     return -1;
@@ -225,16 +227,21 @@ _theora_granule_frame (GstTheoraDec * dec, gint64 granulepos)
 
   /* granulepos is last ilog bits for counting pframes since last iframe and 
    * bits in front of that for the framenumber of the last iframe. */
-  framecount = granulepos >> ilog;
-  framecount += granulepos - (framecount << ilog);
+  framenum = granulepos >> ilog;
+  framenum += granulepos - (framenum << ilog);
 
-  GST_DEBUG_OBJECT (dec, "framecount=%d, ilog=%u", framecount, ilog);
+  /* This is 0-based for old bitstreams, 1-based for new. Fix up. */
+  if (!dec->is_old_bitstream)
+    framenum -= 1;
 
-  return framecount;
+  GST_DEBUG_OBJECT (dec, "framecount=%d, ilog=%u", framenum, ilog);
+
+  return framenum;
 }
 
+/* Return the frame start time corresponding to this granulepos */
 static GstClockTime
-_theora_granule_time (GstTheoraDec * dec, gint64 granulepos)
+_theora_granule_start_time (GstTheoraDec * dec, gint64 granulepos)
 {
   gint64 framecount;
 
@@ -259,7 +266,8 @@ _inc_granulepos (GstTheoraDec * dec, gint64 granulepos)
 
   framecount = _theora_granule_frame (dec, granulepos);
 
-  return (framecount + 1) << dec->granule_shift;
+  return (framecount + 1 +
+      (dec->is_old_bitstream ? 0 : 1)) << dec->granule_shift;
 }
 
 #if 0
@@ -407,7 +415,7 @@ theora_dec_sink_convert (GstPad * pad,
     case GST_FORMAT_DEFAULT:
       switch (*dest_format) {
         case GST_FORMAT_TIME:
-          *dest_value = _theora_granule_time (dec, src_value);
+          *dest_value = _theora_granule_start_time (dec, src_value);
           break;
         default:
           res = FALSE;
@@ -773,6 +781,7 @@ theora_handle_type_packet (GstTheoraDec * dec, ogg_packet * packet)
   GstFlowReturn ret = GST_FLOW_OK;
   gboolean eret;
   GstEvent *event;
+  guint32 bitstream_version;
 
   GST_DEBUG_OBJECT (dec, "fps %d/%d, PAR %d/%d",
       dec->info.fps_numerator, dec->info.fps_denominator,
@@ -823,6 +832,16 @@ theora_handle_type_packet (GstTheoraDec * dec, ogg_packet * packet)
 
   dec->granule_shift = _theora_ilog (dec->info.keyframe_frequency_force - 1);
 
+  /* With libtheora-1.0beta1 the granulepos scheme was changed:
+   * where earlier the granulepos refered to the index/beginning
+   * of a frame, it now refers to the end, which matches the use
+   * in vorbis/speex. We check the bitstream version from the header so
+   * we know which way to interpret the incoming granuepos
+   */
+  bitstream_version = (dec->info.version_major << 16) |
+      (dec->info.version_minor << 8) | dec->info.version_subminor;
+  dec->is_old_bitstream = (bitstream_version <= 0x00030200);
+
   GST_DEBUG_OBJECT (dec, "after fixup frame dimension %dx%d, offset %d:%d",
       dec->width, dec->height, dec->offset_x, dec->offset_y);
 
@@ -1224,9 +1243,9 @@ theora_dec_decode_buffer (GstTheoraDec * dec, GstBuffer * buf)
   if (dec->have_header) {
     if (packet.granulepos != -1) {
       dec->granulepos = packet.granulepos;
-      dec->last_timestamp = _theora_granule_time (dec, packet.granulepos);
+      dec->last_timestamp = _theora_granule_start_time (dec, packet.granulepos);
     } else if (dec->last_timestamp != -1) {
-      dec->last_timestamp = _theora_granule_time (dec, dec->granulepos);
+      dec->last_timestamp = _theora_granule_start_time (dec, dec->granulepos);
     }
     if (dec->last_timestamp == -1 && GST_BUFFER_TIMESTAMP_IS_VALID (buf))
       dec->last_timestamp = GST_BUFFER_TIMESTAMP (buf);
