@@ -50,6 +50,8 @@ struct _GstBaseAudioSinkPrivate
   GstBaseAudioSinkSlaveMethod slave_method;
   /* running average of clock skew */
   GstClockTimeDiff avg_skew;
+  /* the number of samples we aligned last time */
+  gint64 last_align;
 };
 
 /* BaseAudioSink signals and args */
@@ -92,6 +94,7 @@ slave_method_get_type (void)
   static const GEnumValue slave_method[] = {
     {GST_BASE_AUDIO_SINK_SLAVE_RESAMPLE, "Resampling slaving", "resample"},
     {GST_BASE_AUDIO_SINK_SLAVE_SKEW, "Skew slaving", "skew"},
+    {GST_BASE_AUDIO_SINK_SLAVE_NONE, "No slaving", "none"},
     {0, NULL, NULL},
   };
 
@@ -206,6 +209,10 @@ gst_base_audio_sink_class_init (GstBaseAudioSinkClass * klass)
       GST_DEBUG_FUNCPTR (gst_base_audio_sink_async_play);
   gstbasesink_class->activate_pull =
       GST_DEBUG_FUNCPTR (gst_base_audio_sink_activate_pull);
+
+  /* ref class from a thread-safe context to work around missing bit of
+   * thread-safety in GObject */
+  g_type_class_ref (GST_TYPE_AUDIO_CLOCK);
 }
 
 static void
@@ -223,9 +230,9 @@ gst_base_audio_sink_init (GstBaseAudioSink * baseaudiosink,
       (GstAudioClockGetTimeFunc) gst_base_audio_sink_get_time, baseaudiosink);
 
   GST_BASE_SINK (baseaudiosink)->can_activate_push = TRUE;
-  /* FIXME: fix state changes so that both READY_TO_PAUSED and
-     PAUSED_TO_PLAYING return SUCCESS */
-  GST_BASE_SINK (baseaudiosink)->can_activate_pull = TRUE;
+  /* FIXME, enable pull mode when segments, latency, state changes, negotiation
+   * and clock slaving are figured out */
+  GST_BASE_SINK (baseaudiosink)->can_activate_pull = FALSE;
 }
 
 static void
@@ -393,6 +400,97 @@ gst_base_audio_sink_get_time (GstClock * clock, GstBaseAudioSink * sink)
   return result;
 }
 
+/**
+ * gst_base_audio_sink_set_provide_clock:
+ * @sink: a #GstBaseAudioSink
+ * @provide: new state
+ *
+ * Controls whether @sink will provide a clock or not. If @provide is %TRUE, 
+ * gst_element_provide_clock() will return a clock that reflects the datarate
+ * of @sink. If @provide is %FALSE, gst_element_provide_clock() will return NULL.
+ *
+ * Since: 0.10.16
+ */
+void
+gst_base_audio_sink_set_provide_clock (GstBaseAudioSink * sink,
+    gboolean provide)
+{
+  g_return_if_fail (GST_IS_BASE_AUDIO_SINK (sink));
+
+  GST_OBJECT_LOCK (sink);
+  sink->provide_clock = provide;
+  GST_OBJECT_UNLOCK (sink);
+}
+
+/**
+ * gst_base_audio_sink_get_provide_clock:
+ * @sink: a #GstBaseAudioSink
+ *
+ * Queries whether @sink will provide a clock or not. See also
+ * gst_base_audio_sink_set_provide_clock.
+ *
+ * Returns: %TRUE if @sink will provide a clock.
+ *
+ * Since: 0.10.16
+ */
+gboolean
+gst_base_audio_sink_get_provide_clock (GstBaseAudioSink * sink)
+{
+  gboolean result;
+
+  g_return_val_if_fail (GST_IS_BASE_AUDIO_SINK (sink), FALSE);
+
+  GST_OBJECT_LOCK (sink);
+  result = sink->provide_clock;
+  GST_OBJECT_UNLOCK (sink);
+
+  return result;
+}
+
+/**
+ * gst_base_audio_sink_set_slave_method:
+ * @sink: a #GstBaseAudioSink
+ * @method: the new slave method
+ *
+ * Controls how clock slaving will be performed in @sink. 
+ *
+ * Since: 0.10.16
+ */
+void
+gst_base_audio_sink_set_slave_method (GstBaseAudioSink * sink,
+    GstBaseAudioSinkSlaveMethod method)
+{
+  g_return_if_fail (GST_IS_BASE_AUDIO_SINK (sink));
+
+  GST_OBJECT_LOCK (sink);
+  sink->priv->slave_method = method;
+  GST_OBJECT_UNLOCK (sink);
+}
+
+/**
+ * gst_base_audio_sink_get_slave_method:
+ * @sink: a #GstBaseAudioSink
+ *
+ * Get the current slave method used by @sink.
+ *
+ * Returns: The current slave method used by @sink.
+ *
+ * Since: 0.10.16
+ */
+GstBaseAudioSinkSlaveMethod
+gst_base_audio_sink_get_slave_method (GstBaseAudioSink * sink)
+{
+  GstBaseAudioSinkSlaveMethod result;
+
+  g_return_val_if_fail (GST_IS_BASE_AUDIO_SINK (sink), -1);
+
+  GST_OBJECT_LOCK (sink);
+  result = sink->priv->slave_method;
+  GST_OBJECT_UNLOCK (sink);
+
+  return result;
+}
+
 static void
 gst_base_audio_sink_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec)
@@ -409,12 +507,10 @@ gst_base_audio_sink_set_property (GObject * object, guint prop_id,
       sink->latency_time = g_value_get_int64 (value);
       break;
     case PROP_PROVIDE_CLOCK:
-      GST_OBJECT_LOCK (sink);
-      sink->provide_clock = g_value_get_boolean (value);
-      GST_OBJECT_UNLOCK (sink);
+      gst_base_audio_sink_set_provide_clock (sink, g_value_get_boolean (value));
       break;
     case PROP_SLAVE_METHOD:
-      sink->priv->slave_method = g_value_get_enum (value);
+      gst_base_audio_sink_set_slave_method (sink, g_value_get_enum (value));
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -438,12 +534,10 @@ gst_base_audio_sink_get_property (GObject * object, guint prop_id,
       g_value_set_int64 (value, sink->latency_time);
       break;
     case PROP_PROVIDE_CLOCK:
-      GST_OBJECT_LOCK (sink);
-      g_value_set_boolean (value, sink->provide_clock);
-      GST_OBJECT_UNLOCK (sink);
+      g_value_set_boolean (value, gst_base_audio_sink_get_provide_clock (sink));
       break;
     case PROP_SLAVE_METHOD:
-      g_value_set_enum (value, sink->priv->slave_method);
+      g_value_set_enum (value, gst_base_audio_sink_get_slave_method (sink));
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -545,45 +639,52 @@ gst_base_audio_sink_get_times (GstBaseSink * bsink, GstBuffer * buffer,
   *end = GST_CLOCK_TIME_NONE;
 }
 
-/* FIXME, this waits for the drain to happen but it cannot be
- * canceled.
- */
+/* This waits for the drain to happen and can be canceled */
 static gboolean
 gst_base_audio_sink_drain (GstBaseAudioSink * sink)
 {
+  GstClockTime base_time;
+
   if (!sink->ringbuffer)
     return TRUE;
   if (!sink->ringbuffer->spec.rate)
     return TRUE;
 
   /* need to start playback before we can drain, but only when
-   * we have successfully negotiated a format and thus aqcuired the
+   * we have successfully negotiated a format and thus acquired the
    * ringbuffer. */
   if (gst_ring_buffer_is_acquired (sink->ringbuffer))
     gst_ring_buffer_start (sink->ringbuffer);
 
   if (sink->next_sample != -1) {
     GstClockTime time;
-    GstClock *clock;
 
+    /* convert next expected sample to time */
     time =
         gst_util_uint64_scale_int (sink->next_sample, GST_SECOND,
         sink->ringbuffer->spec.rate);
 
+    GST_DEBUG_OBJECT (sink,
+        "last sample %" G_GUINT64_FORMAT ", time %" GST_TIME_FORMAT,
+        sink->next_sample, GST_TIME_ARGS (time));
+
+    /* our time already includes the base_time, _wait_eos() wants a running_time
+     * so we have to subtract the base_time again here. FIXME, store an
+     * unadjusted EOS time so that we don't have to do this. */
     GST_OBJECT_LOCK (sink);
-    if ((clock = GST_ELEMENT_CLOCK (sink)) != NULL) {
-      GstClockID id = gst_clock_new_single_shot_id (clock, time);
+    base_time = GST_ELEMENT_CAST (sink)->base_time;
+    GST_OBJECT_UNLOCK (sink);
 
-      GST_OBJECT_UNLOCK (sink);
+    if (time > base_time)
+      time -= base_time;
+    else
+      time = 0;
 
-      GST_DEBUG_OBJECT (sink, "waiting for last sample to play");
-      gst_clock_id_wait (id, NULL);
+    /* wait for the EOS time to be reached, this is the time when the last
+     * sample is played. */
+    gst_base_sink_wait_eos (GST_BASE_SINK (sink), time, NULL);
 
-      gst_clock_id_unref (id);
-      sink->next_sample = -1;
-    } else {
-      GST_OBJECT_UNLOCK (sink);
-    }
+    sink->next_sample = -1;
   }
   return TRUE;
 }
@@ -754,7 +855,9 @@ gst_base_audio_sink_skew_slaving (GstBaseAudioSink * sink,
 {
   GstClockTime cinternal, cexternal, crate_num, crate_denom;
   GstClockTime etime, itime;
-  GstClockTimeDiff skew, segtime;
+  GstClockTimeDiff skew, segtime, segtime2;
+  gint segsamples;
+  gint64 last_align;
 
   /* get calibration parameters to compensate for offsets */
   gst_clock_get_calibration (sink->provided_clock, &cinternal, &cexternal,
@@ -782,26 +885,55 @@ gst_base_audio_sink_skew_slaving (GstBaseAudioSink * sink,
 
   /* the max drift we allow is the length of a segment */
   segtime = sink->ringbuffer->spec.latency_time * 1000;
+  segtime2 = segtime / 2;
 
   /* adjust playout pointer based on skew */
-  if (sink->priv->avg_skew > segtime) {
+  if (sink->priv->avg_skew > segtime2) {
     /* master is running slower, move internal time forward */
     GST_WARNING_OBJECT (sink,
         "correct clock skew %" G_GINT64_FORMAT " > %" G_GINT64_FORMAT,
-        sink->priv->avg_skew, segtime);
+        sink->priv->avg_skew, segtime2);
     cinternal += segtime;
     sink->priv->avg_skew -= segtime;
-    sink->next_sample = -1;
+
+    segsamples =
+        sink->ringbuffer->spec.segsize /
+        sink->ringbuffer->spec.bytes_per_sample;
+    last_align = sink->priv->last_align;
+
+    /* if we were aligning in the wrong direction or we aligned more than what we
+     * will correct, resync */
+    if (last_align < 0 || last_align > segsamples)
+      sink->next_sample = -1;
+
+    GST_DEBUG_OBJECT (sink,
+        "last_align %" G_GINT64_FORMAT " segsamples %u, next %"
+        G_GUINT64_FORMAT, last_align, segsamples, sink->next_sample);
+
     gst_clock_set_calibration (sink->provided_clock, cinternal, cexternal,
         crate_num, crate_denom);
-  } else if (sink->priv->avg_skew < -segtime) {
+  } else if (sink->priv->avg_skew < -segtime2) {
     /* master is running faster, move external time forwards */
     GST_WARNING_OBJECT (sink,
         "correct clock skew %" G_GINT64_FORMAT " < %" G_GINT64_FORMAT,
-        sink->priv->avg_skew, -segtime);
+        sink->priv->avg_skew, -segtime2);
     cexternal += segtime;
     sink->priv->avg_skew += segtime;
-    sink->next_sample = -1;
+
+    segsamples =
+        sink->ringbuffer->spec.segsize /
+        sink->ringbuffer->spec.bytes_per_sample;
+    last_align = sink->priv->last_align;
+
+    /* if we were aligning in the wrong direction or we aligned more than what we
+     * will correct, resync */
+    if (last_align > 0 || -last_align > segsamples)
+      sink->next_sample = -1;
+
+    GST_DEBUG_OBJECT (sink,
+        "last_align %" G_GINT64_FORMAT " segsamples %u, next %"
+        G_GUINT64_FORMAT, last_align, segsamples, sink->next_sample);
+
     gst_clock_set_calibration (sink->provided_clock, cinternal, cexternal,
         crate_num, crate_denom);
   }
@@ -816,6 +948,28 @@ gst_base_audio_sink_skew_slaving (GstBaseAudioSink * sink,
   *srender_stop = render_stop;
 }
 
+/* apply the clock offset but do no slaving otherwise */
+static void
+gst_base_audio_sink_none_slaving (GstBaseAudioSink * sink,
+    GstClockTime render_start, GstClockTime render_stop,
+    GstClockTime * srender_start, GstClockTime * srender_stop)
+{
+  GstClockTime cinternal, cexternal, crate_num, crate_denom;
+
+  /* get calibration parameters to compensate for offsets */
+  gst_clock_get_calibration (sink->provided_clock, &cinternal, &cexternal,
+      &crate_num, &crate_denom);
+
+  /* convert, ignoring speed */
+  render_start = clock_convert_external (render_start, cinternal, cexternal,
+      crate_num, crate_denom, sink->priv->us_latency);
+  render_stop = clock_convert_external (render_stop, cinternal, cexternal,
+      crate_num, crate_denom, sink->priv->us_latency);
+
+  *srender_start = render_start;
+  *srender_stop = render_stop;
+}
+
 /* converts render_start and render_stop to their slaved values */
 static void
 gst_base_audio_sink_handle_slaving (GstBaseAudioSink * sink,
@@ -831,6 +985,10 @@ gst_base_audio_sink_handle_slaving (GstBaseAudioSink * sink,
       gst_base_audio_sink_skew_slaving (sink, render_start, render_stop,
           srender_start, srender_stop);
       break;
+    case GST_BASE_AUDIO_SINK_SLAVE_NONE:
+      gst_base_audio_sink_none_slaving (sink, render_start, render_stop,
+          srender_start, srender_stop);
+      break;
     default:
       g_warning ("unknown slaving method %d", sink->priv->slave_method);
       break;
@@ -851,7 +1009,7 @@ gst_base_audio_sink_render (GstBaseSink * bsink, GstBuffer * buf)
   gint bps;
   gint accum;
   gint out_samples;
-  GstClockTime base_time = -1, latency;
+  GstClockTime base_time = GST_CLOCK_TIME_NONE, latency;
   GstClock *clock;
   gboolean sync, slaved, align_next;
 
@@ -973,8 +1131,7 @@ gst_base_audio_sink_render (GstBaseSink * bsink, GstBuffer * buf)
       "after latency: start %" GST_TIME_FORMAT " - stop %" GST_TIME_FORMAT,
       GST_TIME_ARGS (render_start), GST_TIME_ARGS (render_stop));
 
-  slaved = clock != sink->provided_clock;
-  if (slaved) {
+  if ((slaved = clock != sink->provided_clock)) {
     /* handle clock slaving */
     gst_base_audio_sink_handle_slaving (sink, render_start, render_stop,
         &render_start, &render_stop);
@@ -1016,11 +1173,11 @@ gst_base_audio_sink_render (GstBaseSink * bsink, GstBuffer * buf)
    * and sample offset position. We always resync if we got a discont anyway and
    * non-discont should be aligned by definition. */
   if (G_LIKELY (diff < ringbuf->spec.rate / DIFF_TOLERANCE)) {
-    GST_DEBUG_OBJECT (sink,
-        "align with prev sample, %" G_GINT64_FORMAT " < %d", diff,
-        ringbuf->spec.rate / DIFF_TOLERANCE);
     /* calc align with previous sample */
     align = sink->next_sample - sample_offset;
+    GST_DEBUG_OBJECT (sink,
+        "align with prev sample, ABS (%" G_GINT64_FORMAT ") < %d", align,
+        ringbuf->spec.rate / DIFF_TOLERANCE);
   } else {
     /* bring sample diff to seconds for error message */
     diff = gst_util_uint64_scale_int (diff, GST_SECOND, ringbuf->spec.rate);
@@ -1033,6 +1190,7 @@ gst_base_audio_sink_render (GstBaseSink * bsink, GstBuffer * buf)
             GST_TIME_ARGS (diff)));
     align = 0;
   }
+  sink->priv->last_align = align;
 
   /* apply alignment */
   render_start += align;
@@ -1258,6 +1416,7 @@ gst_base_audio_sink_async_play (GstBaseSink * basesink)
   itime = gst_clock_get_internal_time (sink->provided_clock);
 
   sink->priv->avg_skew = -1;
+  sink->next_sample = -1;
 
   GST_DEBUG_OBJECT (sink,
       "internal time: %" GST_TIME_FORMAT " external time: %" GST_TIME_FORMAT,
@@ -1314,6 +1473,7 @@ gst_base_audio_sink_change_state (GstElement * element,
       break;
     case GST_STATE_CHANGE_READY_TO_PAUSED:
       sink->next_sample = -1;
+      sink->priv->last_align = -1;
       gst_ring_buffer_set_flushing (sink->ringbuffer, FALSE);
       gst_ring_buffer_may_start (sink->ringbuffer, FALSE);
       break;
@@ -1349,6 +1509,11 @@ gst_base_audio_sink_change_state (GstElement * element,
       gst_ring_buffer_release (sink->ringbuffer);
       break;
     case GST_STATE_CHANGE_READY_TO_NULL:
+      /* we release again here because the aqcuire happens when setting the
+       * caps, which happens before we commit the state to PAUSED and thus the
+       * PAUSED->READY state change (see above, where we release the ringbuffer)
+       * might not be called when we get here. */
+      gst_ring_buffer_release (sink->ringbuffer);
       gst_ring_buffer_close_device (sink->ringbuffer);
       break;
     default:
