@@ -286,11 +286,6 @@ gst_ring_buffer_parse_caps (GstRingBufferSpec * spec, GstCaps * caps)
   /* we have to differentiate between int and float formats */
   mimetype = gst_structure_get_name (structure);
 
-  /* get rate and channels */
-  if (!(gst_structure_get_int (structure, "rate", &spec->rate) &&
-          gst_structure_get_int (structure, "channels", &spec->channels)))
-    goto parse_error;
-
   if (!strncmp (mimetype, "audio/x-raw-int", 15)) {
     gint endianness;
     const FormatDef *def;
@@ -299,7 +294,9 @@ gst_ring_buffer_parse_caps (GstRingBufferSpec * spec, GstCaps * caps)
     spec->type = GST_BUFTYPE_LINEAR;
 
     /* extract the needed information from the cap */
-    if (!(gst_structure_get_int (structure, "width", &spec->width) &&
+    if (!(gst_structure_get_int (structure, "rate", &spec->rate) &&
+            gst_structure_get_int (structure, "channels", &spec->channels) &&
+            gst_structure_get_int (structure, "width", &spec->width) &&
             gst_structure_get_int (structure, "depth", &spec->depth) &&
             gst_structure_get_boolean (structure, "signed", &spec->sign)))
       goto parse_error;
@@ -333,8 +330,10 @@ gst_ring_buffer_parse_caps (GstRingBufferSpec * spec, GstCaps * caps)
 
     spec->type = GST_BUFTYPE_FLOAT;
 
-    /* get layout */
-    if (!gst_structure_get_int (structure, "width", &spec->width))
+    /* extract the needed information from the cap */
+    if (!(gst_structure_get_int (structure, "rate", &spec->rate) &&
+            gst_structure_get_int (structure, "channels", &spec->channels) &&
+            gst_structure_get_int (structure, "width", &spec->width)))
       goto parse_error;
 
     /* match layout to format wrt to endianness */
@@ -353,6 +352,11 @@ gst_ring_buffer_parse_caps (GstRingBufferSpec * spec, GstCaps * caps)
     /* float silence is all zeros.. */
     memset (spec->silence_sample, 0, 32);
   } else if (!strncmp (mimetype, "audio/x-alaw", 12)) {
+    /* extract the needed information from the cap */
+    if (!(gst_structure_get_int (structure, "rate", &spec->rate) &&
+            gst_structure_get_int (structure, "channels", &spec->channels)))
+      goto parse_error;
+
     spec->type = GST_BUFTYPE_A_LAW;
     spec->format = GST_A_LAW;
     spec->width = 8;
@@ -360,12 +364,37 @@ gst_ring_buffer_parse_caps (GstRingBufferSpec * spec, GstCaps * caps)
     for (i = 0; i < spec->channels; i++)
       spec->silence_sample[i] = 0xd5;
   } else if (!strncmp (mimetype, "audio/x-mulaw", 13)) {
+    /* extract the needed information from the cap */
+    if (!(gst_structure_get_int (structure, "rate", &spec->rate) &&
+            gst_structure_get_int (structure, "channels", &spec->channels)))
+      goto parse_error;
+
     spec->type = GST_BUFTYPE_MU_LAW;
     spec->format = GST_MU_LAW;
     spec->width = 8;
     spec->depth = 8;
     for (i = 0; i < spec->channels; i++)
       spec->silence_sample[i] = 0xff;
+  } else if (!strncmp (mimetype, "audio/x-iec958", 14)) {
+    /* extract the needed information from the cap */
+    if (!(gst_structure_get_int (structure, "rate", &spec->rate)))
+      goto parse_error;
+
+    spec->type = GST_BUFTYPE_IEC958;
+    spec->format = GST_IEC958;
+    spec->width = 16;
+    spec->depth = 16;
+    spec->channels = 2;
+  } else if (!strncmp (mimetype, "audio/x-ac3", 11)) {
+    /* extract the needed information from the cap */
+    if (!(gst_structure_get_int (structure, "rate", &spec->rate)))
+      goto parse_error;
+
+    spec->type = GST_BUFTYPE_AC3;
+    spec->format = GST_AC3;
+    spec->width = 16;
+    spec->depth = 16;
+    spec->channels = 2;
   } else {
     goto parse_error;
   }
