@@ -90,6 +90,10 @@ typedef enum {
  * @GST_BUFTYPE_IMA_ADPCM: samples in ima adpcm
  * @GST_BUFTYPE_MPEG: samples in mpeg audio format
  * @GST_BUFTYPE_GSM: samples in gsm format
+ * @GST_BUFTYPE_IEC958: samples in IEC958 frames (e.g. AC3)
+ * @GST_BUFTYPE_AC3: samples in AC3 format
+ * @GST_BUFTYPE_EAC3: samples in EAC3 format
+ * @GST_BUFTYPE_DTS: samples in DTS format
  *
  * The format of the samples in the ringbuffer.
  */
@@ -101,7 +105,11 @@ typedef enum
   GST_BUFTYPE_A_LAW,
   GST_BUFTYPE_IMA_ADPCM,
   GST_BUFTYPE_MPEG,
-  GST_BUFTYPE_GSM
+  GST_BUFTYPE_GSM,
+  GST_BUFTYPE_IEC958,
+  GST_BUFTYPE_AC3,
+  GST_BUFTYPE_EAC3,
+  GST_BUFTYPE_DTS
 } GstBufferFormatType;
 
 typedef enum
@@ -149,7 +157,11 @@ typedef enum
   GST_A_LAW,
   GST_IMA_ADPCM,
   GST_MPEG,
-  GST_GSM
+  GST_GSM,
+  GST_IEC958,
+  GST_AC3,
+  GST_EAC3,
+  GST_DTS
 } GstBufferFormat;
 
 /**
