@@ -43,7 +43,6 @@
 #include "gst/gst-i18n-plugin.h"
 
 #include "pbutils.h"
-#include "pbutils-private.h"
 
 #include <string.h>
 
@@ -62,28 +61,18 @@ typedef struct
 
 static const FormatInfo formats[] = {
   /* container/tag formats with static descriptions */
-  {"application/gxf", "General Exchange Format (GXF)", FLAG_CONTAINER},
   {"application/ogg", "Ogg", FLAG_CONTAINER},
-  {"application/mxf", "Material eXchange Format (MXF)", FLAG_CONTAINER},
   {"application/vnd.rn-realmedia", "Realmedia", FLAG_CONTAINER},
-  {"application/x-annodex", "Ogg", FLAG_CONTAINER},
   {"application/x-id3", N_("ID3 tag"), FLAG_CONTAINER},
   {"application/x-ape", N_("APE tag"), FLAG_CONTAINER},
-  {"application/x-apetag", N_("APE tag"), FLAG_CONTAINER},
   {"application/x-icy", N_("ICY internet radio"), FLAG_CONTAINER},
-  {"application/x-3gp", "3GP", FLAG_CONTAINER},
-  {"application/x-pn-realaudio", "RealAudio", FLAG_CONTAINER},
-  {"application/x-yuv4mpeg", "Y4M", FLAG_CONTAINER},
-  {"multipart/x-mixed-replace", "Multipart", FLAG_CONTAINER},
   {"video/x-fli", "FLI/FLC/FLX Animation", FLAG_CONTAINER},
   {"video/x-flv", "Flash", FLAG_CONTAINER},
   {"video/x-matroska", "Matroska", FLAG_CONTAINER},
-  {"video/webm", "WebM", FLAG_CONTAINER},
   {"video/x-ms-asf", "Advanced Streaming Format (ASF)", FLAG_CONTAINER},
-  {"video/x-msvideo", "Audio Video Interleave (AVI)", FLAG_CONTAINER},
+  {"video/x-msvideo", "AVI", FLAG_CONTAINER},
   {"video/x-quicktime", "Quicktime", FLAG_CONTAINER},
   {"video/quicktime", "Quicktime", FLAG_CONTAINER},
-  {"video/mj2", "Motion JPEG 2000", FLAG_CONTAINER},
 
   /* audio formats with static descriptions */
   {"audio/x-ac3", "AC-3 (ATSC A/52)", 0},
@@ -103,7 +92,6 @@ static const FormatInfo formats[] = {
   {"audio/x-amr-nb-sh", "Adaptive Multi Rate NarrowBand (AMR-NB)", 0},
   {"audio/x-amr-wb-sh", "Adaptive Multi Rate WideBand (AMR-WB)", 0},
   {"audio/x-au", "Sun .au", 0},
-  {"audio/x-celt", "Constrained Energy Lapped Transform (CELT)", 0},
   {"audio/x-cinepak", "Cinepak Audio", 0},
   {"audio/x-dpcm", "DPCM", 0},
   {"audio/x-dts", "DTS", 0},
@@ -124,14 +112,12 @@ static const FormatInfo formats[] = {
   {"audio/x-nist", "Sphere NIST", 0},
   {"audio/x-nsf", "Nintendo NSF", 0},
   {"audio/x-paris", "Ensoniq PARIS", 0},
-  {"audio/x-qdm", "QDesign Music (QDM)", 0},
   {"audio/x-qdm2", "QDesign Music (QDM) 2", 0},
   {"audio/x-ralf-mpeg4-generic", "Real Audio Lossless (RALF)", 0},
   {"audio/x-sds", "SDS", 0},
   {"audio/x-shorten", "Shorten Lossless", 0},
   {"audio/x-sid", "Sid", 0},
   {"audio/x-sipro", "Sipro/ACELP.NET Voice", 0},
-  {"audio/x-siren", "Siren", 0},
   {"audio/x-spc", "SNES-SPC700 Sound File Data", 0},
   {"audio/x-speex", "Speex", 0},
   {"audio/x-svx", "Amiga IFF / SVX8 / SV16", 0},
@@ -152,28 +138,23 @@ static const FormatInfo formats[] = {
   {"video/sp5x", "Sunplus JPEG 5.x", 0},
   {"video/vivo", "Vivo", 0},
   {"video/x-3ivx", "3ivx", 0},
-  {"video/x-4xm", "4X Technologies Video", 0},
+  {"video/x-4xm", "4X Techologies Video", 0},
   {"video/x-apple-video", "Apple video", 0},
-  {"video/x-aasc", "Autodesk Animator", 0},
   {"video/x-camtasia", "TechSmith Camtasia", 0},
   {"video/x-cdxa", "RIFF/CDXA (VCD)", 0},
   {"video/x-cinepak", "Cinepak Video", 0},
   {"video/x-cirrus-logic-accupak", "Cirrus Logipak AccuPak", 0},
   {"video/x-compressed-yuv", N_("CYUV Lossless"), 0},
   {"video/x-dirac", "Dirac", 0},
-  {"video/x-dnxhd", "Digital Nonlinear Extensible High Definition (DNxHD)", 0},
-  /* FIXME 0.11: rename to subpicture/x-dvd or so */
   {"video/x-dvd-subpicture", "DVD subpicture", 0},
   {"video/x-ffv", N_("FFMpeg v1"), 0},
   {"video/x-flash-screen", "Flash Screen Video", 0},
-  {"video/x-flash-video", "Sorenson Spark Video", 0},
+  {"video/x-flash-video", "Flash Video", 0},
   {"video/x-h261", "H.261", 0},
   {"video/x-huffyuv", "Huffyuv", 0},
   {"video/x-intel-h263", "Intel H.263", 0},
   {"video/x-jpeg", "Motion JPEG", 0},
   /* { "video/x-jpeg-b", "", 0 }, does this actually exist? */
-  {"video/x-loco", "LOCO Lossless", 0},
-  {"video/x-mimic", "MIMIC", 0},
   {"video/x-mjpeg", "Motion-JPEG", 0},
   {"video/x-mjpeg-b", "Motion-JPEG format B", 0},
   {"video/mpegts", "MPEG-2 Transport Stream", FLAG_CONTAINER},
@@ -197,12 +178,9 @@ static const FormatInfo formats[] = {
   {"video/x-vp5", "On2 VP5", 0},
   {"video/x-vp6", "On2 VP6", 0},
   {"video/x-vp6-flash", "On2 VP6/Flash", 0},
-  {"video/x-vp6-alpha", "On2 VP6 with alpha", 0},
   {"video/x-vp7", "On2 VP7", 0},
-  {"video/x-vp8", "VP8", 0},
   {"video/x-xvid", "XVID MPEG-4", 0},
   {"video/x-zlib", "Lossless zlib video", 0},
-  {"video/x-zmbv", "Zip Motion Block video", 0},
 
   /* image formats with static descriptions */
   {"image/bmp", "BMP", 0},
@@ -212,40 +190,24 @@ static const FormatInfo formats[] = {
   {"image/jpeg", "JPEG", 0},
   {"image/jng", "JPEG Network Graphics (JNG)", 0},
   {"image/png", "PNG", 0},
-  {"image/pbm", "Portable BitMap (PBM)", 0},
-  {"image/ppm", "Portable PixMap (PPM)", 0},
+  {"image/pbm", "PBM", 0},
+  {"image/ppm", "PPM", 0},
   {"image/svg+xml", "Scalable Vector Graphics (SVG)", 0},
   {"image/tiff", "TIFF", 0},
   {"image/x-cmu-raster", "CMU Raster Format", 0},
-  {"image/x-degas", "DEGAS", 0},
   {"image/x-icon", "ICO", 0},
-  {"image/x-j2c", "JPEG 2000", 0},
-  {"image/x-jpc", "JPEG 2000", 0},
-  {"image/jp2", "JPEG 2000", 0},
-  {"image/x-pcx", "PCX", 0},
   {"image/x-xcf", "XFC", 0},
   {"image/x-pixmap", "XPM", 0},
-  {"image/x-portable-anymap", "Portable AnyMap (PAM)", 0},
-  {"image/x-portable-graymap", "Portable GrayMap (PGM)", 0},
   {"image/x-xpixmap", "XPM", 0},
   {"image/x-quicktime", "QuickTime Image Format (QTIF)", 0},
   {"image/x-sun-raster", "Sun Raster Format (RAS)", 0},
   {"image/x-tga", "TGA", 0},
-  {"image/vnd.wap.wbmp", "Wireless Bitmap", 0},
 
   /* subtitle formats with static descriptions */
-  {"application/x-ass", "ASS", 0},
   {"application/x-subtitle-sami", N_("Sami subtitle format"), 0},
   {"application/x-subtitle-tmplayer", N_("TMPlayer subtitle format"), 0},
-  {"application/x-kate", "Kate", 0},
-  {"subtitle/x-kate", N_("Kate subtitle format"), 0},
-  {"subpicture/x-dvb", "DVB subtitles", 0},
   /* add variant field to typefinder? { "application/x-subtitle", N_("subtitle"), 0}, */
 
-  /* non-audio/video/container formats */
-  {"hdv/aux-v", "HDV AUX-V", 0},
-  {"hdv/aux-a", "HDV AUX-A", 0},
-
   /* formats with dynamic descriptions */
   {"audio/mpeg", NULL, 0},
   {"audio/x-adpcm", NULL, 0},
@@ -385,8 +347,6 @@ format_info_get_desc (const FormatInfo * info, const GstCaps * caps)
       ret = "ITU H.264";
     else if (strcmp (variant, "videosoft") == 0)
       ret = "Videosoft H.264";
-    else if (strcmp (variant, "lead") == 0)
-      ret = "Lead H.264";
     else {
       GST_WARNING ("Unknown H264 variant '%s'", variant);
       ret = "H.264";
@@ -793,27 +753,23 @@ gchar *
 gst_pb_utils_get_decoder_description (const GstCaps * caps)
 {
   gchar *str, *ret;
-  GstCaps *tmp;
 
   g_return_val_if_fail (caps != NULL, NULL);
   g_return_val_if_fail (GST_IS_CAPS (caps), NULL);
-
-  tmp = copy_and_clean_caps (caps);
-
-  g_return_val_if_fail (gst_caps_is_fixed (tmp), NULL);
+  g_return_val_if_fail (gst_caps_is_fixed (caps), NULL);
 
   /* special-case RTP caps */
-  if (caps_are_rtp_caps (tmp, "video", &str)) {
+  if (caps_are_rtp_caps (caps, "video", &str)) {
     ret = g_strdup_printf (_("%s video RTP depayloader"), str);
-  } else if (caps_are_rtp_caps (tmp, "audio", &str)) {
+  } else if (caps_are_rtp_caps (caps, "audio", &str)) {
     ret = g_strdup_printf (_("%s audio RTP depayloader"), str);
-  } else if (caps_are_rtp_caps (tmp, "application", &str)) {
+  } else if (caps_are_rtp_caps (caps, "application", &str)) {
     ret = g_strdup_printf (_("%s RTP depayloader"), str);
   } else {
     const FormatInfo *info;
 
-    str = gst_pb_utils_get_codec_description (tmp);
-    info = find_format_info (tmp);
+    str = gst_pb_utils_get_codec_description (caps);
+    info = find_format_info (caps);
     if (info != NULL && (info->flags & FLAG_CONTAINER) != 0) {
       ret = g_strdup_printf (_("%s demuxer"), str);
     } else {
@@ -822,7 +778,6 @@ gst_pb_utils_get_decoder_description (const GstCaps * caps)
   }
 
   g_free (str);
-  gst_caps_unref (tmp);
 
   return ret;
 }
@@ -846,25 +801,23 @@ gchar *
 gst_pb_utils_get_encoder_description (const GstCaps * caps)
 {
   gchar *str, *ret;
-  GstCaps *tmp;
 
   g_return_val_if_fail (caps != NULL, NULL);
   g_return_val_if_fail (GST_IS_CAPS (caps), NULL);
-  tmp = copy_and_clean_caps (caps);
-  g_return_val_if_fail (gst_caps_is_fixed (tmp), NULL);
+  g_return_val_if_fail (gst_caps_is_fixed (caps), NULL);
 
   /* special-case RTP caps */
-  if (caps_are_rtp_caps (tmp, "video", &str)) {
+  if (caps_are_rtp_caps (caps, "video", &str)) {
     ret = g_strdup_printf (_("%s video RTP payloader"), str);
-  } else if (caps_are_rtp_caps (tmp, "audio", &str)) {
+  } else if (caps_are_rtp_caps (caps, "audio", &str)) {
     ret = g_strdup_printf (_("%s audio RTP payloader"), str);
-  } else if (caps_are_rtp_caps (tmp, "application", &str)) {
+  } else if (caps_are_rtp_caps (caps, "application", &str)) {
     ret = g_strdup_printf (_("%s RTP payloader"), str);
   } else {
     const FormatInfo *info;
 
-    str = gst_pb_utils_get_codec_description (tmp);
-    info = find_format_info (tmp);
+    str = gst_pb_utils_get_codec_description (caps);
+    info = find_format_info (caps);
     if (info != NULL && (info->flags & FLAG_CONTAINER) != 0) {
       ret = g_strdup_printf (_("%s muxer"), str);
     } else {
@@ -873,7 +826,6 @@ gst_pb_utils_get_encoder_description (const GstCaps * caps)
   }
 
   g_free (str);
-  gst_caps_unref (tmp);
 
   return ret;
 }
@@ -963,19 +915,17 @@ gst_pb_utils_get_codec_description (const GstCaps * caps)
 {
   const FormatInfo *info;
   gchar *str, *comma;
-  GstCaps *tmp;
 
   g_return_val_if_fail (caps != NULL, NULL);
   g_return_val_if_fail (GST_IS_CAPS (caps), NULL);
-  tmp = copy_and_clean_caps (caps);
-  g_return_val_if_fail (gst_caps_is_fixed (tmp), NULL);
+  g_return_val_if_fail (gst_caps_is_fixed (caps), NULL);
 
-  info = find_format_info (tmp);
+  info = find_format_info (caps);
 
   if (info) {
-    str = format_info_get_desc (info, tmp);
+    str = format_info_get_desc (info, caps);
   } else {
-    str = gst_caps_to_string (tmp);
+    str = gst_caps_to_string (caps);
 
     /* cut off everything after the media type, if there is anything */
     if ((comma = strchr (str, ','))) {
@@ -987,7 +937,6 @@ gst_pb_utils_get_codec_description (const GstCaps * caps)
 
     GST_WARNING ("No description available for media type: %s", str);
   }
-  gst_caps_unref (tmp);
 
   return str;
 }
