@@ -31,6 +31,15 @@
 
 #include "gstbasertpdepayload.h"
 
+#ifdef GST_DISABLE_DEPRECATED
+#define QUEUE_LOCK_INIT(base)   (g_static_rec_mutex_init(&base->queuelock))
+#define QUEUE_LOCK_FREE(base)   (g_static_rec_mutex_free(&base->queuelock))
+#define QUEUE_LOCK(base)        (g_static_rec_mutex_lock(&base->queuelock))
+#define QUEUE_UNLOCK(base)      (g_static_rec_mutex_unlock(&base->queuelock))
+#else
+/* otherwise it's already been defined in the header (FIXME 0.11)*/
+#endif
+
 GST_DEBUG_CATEGORY_STATIC (basertpdepayload_debug);
 #define GST_CAT_DEFAULT (basertpdepayload_debug)
 
@@ -39,15 +48,14 @@ GST_DEBUG_CATEGORY_STATIC (basertpdepayload_debug);
 
 struct _GstBaseRTPDepayloadPrivate
 {
-  guint64 clock_base;
-
   GstClockTime npt_start;
   GstClockTime npt_stop;
   gdouble play_speed;
   gdouble play_scale;
 
-  GstClockTime ts_wraparound;
-  GstClockTime prev_timestamp;
+  gboolean discont;
+  GstClockTime timestamp;
+  GstClockTime duration;
 };
 
 /* Filter signals and args */
@@ -62,7 +70,7 @@ enum
 enum
 {
   PROP_0,
-  PROP_QUEUE_DELAY,
+  PROP_QUEUE_DELAY
 };
 
 static void gst_base_rtp_depayload_finalize (GObject * object);
@@ -80,14 +88,8 @@ static gboolean gst_base_rtp_depayload_handle_sink_event (GstPad * pad,
 static GstStateChangeReturn gst_base_rtp_depayload_change_state (GstElement *
     element, GstStateChange transition);
 
-static GstFlowReturn gst_base_rtp_depayload_add_to_queue (GstBaseRTPDepayload *
-    filter, GstBuffer * in);
-static GstFlowReturn gst_base_rtp_depayload_process (GstBaseRTPDepayload *
-    filter, GstBuffer * rtp_buf);
 static void gst_base_rtp_depayload_set_gst_timestamp
-    (GstBaseRTPDepayload * filter, guint32 timestamp, GstBuffer * buf);
-static void gst_base_rtp_depayload_wait (GstBaseRTPDepayload * filter,
-    GstClockTime time);
+    (GstBaseRTPDepayload * filter, guint32 rtptime, GstBuffer * buf);
 
 GST_BOILERPLATE (GstBaseRTPDepayload, gst_base_rtp_depayload, GstElement,
     GST_TYPE_ELEMENT);
@@ -114,14 +116,23 @@ gst_base_rtp_depayload_class_init (GstBaseRTPDepayloadClass * klass)
   gobject_class->set_property = gst_base_rtp_depayload_set_property;
   gobject_class->get_property = gst_base_rtp_depayload_get_property;
 
+  /**
+   * GstBaseRTPDepayload::queue-delay
+   *
+   * Control the amount of packets to buffer.
+   *
+   * Deprecated: Use a jitterbuffer or RTP session manager to delay packet
+   * playback. This property has no effect anymore since 0.10.15.
+   */
+#ifndef GST_REMOVE_DEPRECATED
   g_object_class_install_property (gobject_class, PROP_QUEUE_DELAY,
-      g_param_spec_uint ("queue_delay", "Queue Delay",
-          "Amount of ms to queue/buffer", 0, G_MAXUINT, DEFAULT_QUEUE_DELAY,
-          G_PARAM_READWRITE));
+      g_param_spec_uint ("queue-delay", "Queue Delay",
+          "Amount of ms to queue/buffer, deprecated", 0, G_MAXUINT,
+          DEFAULT_QUEUE_DELAY, G_PARAM_READWRITE));
+#endif
 
   gstelement_class->change_state = gst_base_rtp_depayload_change_state;
 
-  klass->add_to_queue = gst_base_rtp_depayload_add_to_queue;
   klass->set_gst_timestamp = gst_base_rtp_depayload_set_gst_timestamp;
 
   GST_DEBUG_CATEGORY_INIT (basertpdepayload_debug, "basertpdepayload", 0,
@@ -160,16 +171,15 @@ gst_base_rtp_depayload_init (GstBaseRTPDepayload * filter,
 
   filter->queue = g_queue_new ();
   filter->queue_delay = DEFAULT_QUEUE_DELAY;
+
+  gst_segment_init (&filter->segment, GST_FORMAT_UNDEFINED);
 }
 
 static void
 gst_base_rtp_depayload_finalize (GObject * object)
 {
-  GstBuffer *buf;
   GstBaseRTPDepayload *filter = GST_BASE_RTP_DEPAYLOAD (object);
 
-  while ((buf = g_queue_pop_head (filter->queue)))
-    gst_buffer_unref (buf);
   g_queue_free (filter->queue);
 
   G_OBJECT_CLASS (parent_class)->finalize (object);
@@ -194,19 +204,13 @@ gst_base_rtp_depayload_setcaps (GstPad * pad, GstCaps * caps)
 
   caps_struct = gst_caps_get_structure (caps, 0);
 
-  /* get clock base if any, we need this for the newsegment */
-  value = gst_structure_get_value (caps_struct, "clock-base");
-  if (value && G_VALUE_HOLDS_UINT (value))
-    priv->clock_base = g_value_get_uint (value);
-  else
-    priv->clock_base = -1;
-
   /* get other values for newsegment */
   value = gst_structure_get_value (caps_struct, "npt-start");
   if (value && G_VALUE_HOLDS_UINT64 (value))
     priv->npt_start = g_value_get_uint64 (value);
   else
     priv->npt_start = 0;
+  GST_DEBUG_OBJECT (filter, "NPT start %" G_GUINT64_FORMAT, priv->npt_start);
 
   value = gst_structure_get_value (caps_struct, "npt-stop");
   if (value && G_VALUE_HOLDS_UINT64 (value))
@@ -214,6 +218,8 @@ gst_base_rtp_depayload_setcaps (GstPad * pad, GstCaps * caps)
   else
     priv->npt_stop = -1;
 
+  GST_DEBUG_OBJECT (filter, "NPT stop %" G_GUINT64_FORMAT, priv->npt_stop);
+
   value = gst_structure_get_value (caps_struct, "play-speed");
   if (value && G_VALUE_HOLDS_DOUBLE (value))
     priv->play_speed = g_value_get_double (value);
@@ -226,8 +232,6 @@ gst_base_rtp_depayload_setcaps (GstPad * pad, GstCaps * caps)
   else
     priv->play_scale = 1.0;
 
-  priv->prev_timestamp = -1;
-
   if (bclass->set_caps)
     res = bclass->set_caps (filter, caps);
   else
@@ -242,76 +246,75 @@ static GstFlowReturn
 gst_base_rtp_depayload_chain (GstPad * pad, GstBuffer * in)
 {
   GstBaseRTPDepayload *filter;
+  GstBaseRTPDepayloadPrivate *priv;
   GstBaseRTPDepayloadClass *bclass;
   GstFlowReturn ret = GST_FLOW_OK;
+  GstBuffer *out_buf;
+  GstClockTime timestamp;
 
   filter = GST_BASE_RTP_DEPAYLOAD (GST_OBJECT_PARENT (pad));
 
-  if (filter->clock_rate == 0)
-    goto not_configured;
+  priv = filter->priv;
+  priv->discont = GST_BUFFER_IS_DISCONT (in);
+
+  /* convert to running_time and save the timestamp, this is the timestamp
+   * we put on outgoing buffers. */
+  timestamp = GST_BUFFER_TIMESTAMP (in);
+  timestamp = gst_segment_to_running_time (&filter->segment, GST_FORMAT_TIME,
+      timestamp);
+  priv->timestamp = timestamp;
+  priv->duration = GST_BUFFER_DURATION (in);
 
   bclass = GST_BASE_RTP_DEPAYLOAD_GET_CLASS (filter);
 
-  if (filter->queue_delay == 0) {
-    GST_DEBUG_OBJECT (filter, "Pushing directly!");
-    ret = gst_base_rtp_depayload_process (filter, in);
-  } else {
-    if (bclass->add_to_queue)
-      ret = bclass->add_to_queue (filter, in);
-    else
-      goto no_delay;
-  }
-  return ret;
+  /* let's send it out to processing */
+  out_buf = bclass->process (filter, in);
+  if (out_buf) {
+    guint32 rtptime;
 
-  /* ERRORS */
-not_configured:
-  {
-    GST_ELEMENT_ERROR (filter, STREAM, FORMAT,
-        (NULL), ("no clock rate was specified, likely incomplete input caps"));
-    gst_buffer_unref (in);
-    return GST_FLOW_NOT_NEGOTIATED;
-  }
-no_delay:
-  {
-    GST_ELEMENT_ERROR (filter, STREAM, NOT_IMPLEMENTED,
-        (NULL), ("This element cannot operate with delay"));
-    gst_buffer_unref (in);
-    return GST_FLOW_NOT_SUPPORTED;
+    rtptime = gst_rtp_buffer_get_timestamp (in);
+
+    /* we pass rtptime as backward compatibility, in reality, the incomming
+     * buffer timestamp is always applied to the outgoing packet. */
+    ret = gst_base_rtp_depayload_push_ts (filter, rtptime, out_buf);
   }
+  gst_buffer_unref (in);
+
+  return ret;
 }
 
 static gboolean
 gst_base_rtp_depayload_handle_sink_event (GstPad * pad, GstEvent * event)
 {
-  GstBaseRTPDepayload *filter =
-      GST_BASE_RTP_DEPAYLOAD (GST_OBJECT_PARENT (pad));
+  GstBaseRTPDepayload *filter;
   gboolean res = TRUE;
 
+  filter = GST_BASE_RTP_DEPAYLOAD (GST_OBJECT_PARENT (pad));
+
   switch (GST_EVENT_TYPE (event)) {
+    case GST_EVENT_FLUSH_STOP:
+      res = gst_pad_push_event (filter->srcpad, event);
+
+      gst_segment_init (&filter->segment, GST_FORMAT_UNDEFINED);
+      filter->need_newsegment = TRUE;
+      break;
     case GST_EVENT_NEWSEGMENT:
     {
-      /* intercept NEWSEGMENT events only if the packet scheduler thread
-         is active */
-      if (filter->thread) {
-        GST_DEBUG_OBJECT (filter,
-            "Upstream sent a NEWSEGMENT, handle in worker thread.");
-        /* the worker thread will assign a new RTP-TS<->GST-TS mapping
-         * based on the next processed RTP packet */
-        filter->need_newsegment = TRUE;
-        gst_event_unref (event);
-        break;
-      } else {
-        GstFormat format;
+      gboolean update;
+      gdouble rate;
+      GstFormat fmt;
+      gint64 start, stop, position;
 
-        gst_event_parse_new_segment (event, NULL, NULL, &format, NULL, NULL,
-            NULL);
-        if (format != GST_FORMAT_TIME)
-          goto wrong_format;
+      gst_event_parse_new_segment (event, &update, &rate, &fmt, &start, &stop,
+          &position);
 
-        GST_DEBUG_OBJECT (filter,
-            "Upstream sent a NEWSEGMENT, passing through.");
-      }
-      /* note: pass through to default if no thread running */
+      gst_segment_set_newsegment (&filter->segment, update, rate, fmt,
+          start, stop, position);
+
+      /* don't pass the event downstream, we generate our own segment including
+       * the NTP time and other things we receive in caps */
+      gst_event_unref (event);
+      break;
     }
     default:
       /* pass other events forward */
@@ -319,73 +322,18 @@ gst_base_rtp_depayload_handle_sink_event (GstPad * pad, GstEvent * event)
       break;
   }
   return res;
-
-  /* ERRORS */
-wrong_format:
-  {
-    GST_DEBUG_OBJECT (filter,
-        "Upstream sent a NEWSEGMENT in wrong format, dropping.");
-    gst_event_unref (event);
-    return TRUE;
-  }
-}
-
-static GstFlowReturn
-gst_base_rtp_depayload_add_to_queue (GstBaseRTPDepayload * filter,
-    GstBuffer * in)
-{
-  GQueue *queue = filter->queue;
-  int i;
-
-  /* our first packet, just push it */
-  QUEUE_LOCK (filter);
-  if (g_queue_is_empty (queue)) {
-    g_queue_push_tail (queue, in);
-    QUEUE_UNLOCK (filter);
-  } else {
-    guint16 seqnum, queueseq;
-    guint32 timestamp;
-
-    seqnum = gst_rtp_buffer_get_seq (in);
-    queueseq = gst_rtp_buffer_get_seq (GST_BUFFER (g_queue_peek_head (queue)));
-
-    /* look for right place to insert it */
-    i = 0;
-
-    /* Check for seqnum wraparound.
-     * Seqnums in the lowest quadrant of the 0-65535 space are considered to
-     * be greater than seqnums in the highest quadrant of this space. */
-    while (seqnum > queueseq || (seqnum < 16384 && queueseq > 49150)) {
-      gpointer data;
-
-      i++;
-      data = g_queue_peek_nth (queue, i);
-      if (!data)
-        break;
-
-      queueseq = gst_rtp_buffer_get_seq (GST_BUFFER (data));
-    }
-
-    /* now insert it at that place */
-    g_queue_push_nth (queue, in, i);
-    QUEUE_UNLOCK (filter);
-
-    timestamp = gst_rtp_buffer_get_timestamp (in);
-
-    GST_DEBUG_OBJECT (filter,
-        "Packet added to queue %d at pos %d timestamp %u sn %d",
-        g_queue_get_length (queue), i, timestamp, seqnum);
-  }
-  return GST_FLOW_OK;
 }
 
 static GstFlowReturn
 gst_base_rtp_depayload_push_full (GstBaseRTPDepayload * filter,
-    gboolean do_ts, guint32 timestamp, GstBuffer * out_buf)
+    gboolean do_ts, guint32 rtptime, GstBuffer * out_buf)
 {
   GstFlowReturn ret;
   GstCaps *srccaps;
   GstBaseRTPDepayloadClass *bclass;
+  GstBaseRTPDepayloadPrivate *priv;
+
+  priv = filter->priv;
 
   /* set the caps if any */
   srccaps = GST_PAD_CAPS (filter->srcpad);
@@ -396,7 +344,12 @@ gst_base_rtp_depayload_push_full (GstBaseRTPDepayload * filter,
 
   /* set the timestamp if we must and can */
   if (bclass->set_gst_timestamp && do_ts)
-    bclass->set_gst_timestamp (filter, timestamp, out_buf);
+    bclass->set_gst_timestamp (filter, rtptime, out_buf);
+
+  if (priv->discont) {
+    GST_BUFFER_FLAG_SET (out_buf, GST_BUFFER_FLAG_DISCONT);
+    priv->discont = FALSE;
+  }
 
   /* push it */
   GST_LOG_OBJECT (filter, "Pushing buffer size %d, timestamp %" GST_TIME_FORMAT,
@@ -449,95 +402,32 @@ gst_base_rtp_depayload_push (GstBaseRTPDepayload * filter, GstBuffer * out_buf)
   return gst_base_rtp_depayload_push_full (filter, FALSE, 0, out_buf);
 }
 
-static GstFlowReturn
-gst_base_rtp_depayload_process (GstBaseRTPDepayload * filter,
-    GstBuffer * rtp_buf)
-{
-  GstBaseRTPDepayloadClass *bclass;
-  GstBuffer *out_buf;
-  GstFlowReturn ret = GST_FLOW_OK;
-
-  bclass = GST_BASE_RTP_DEPAYLOAD_GET_CLASS (filter);
-
-  /* let's send it out to processing */
-  out_buf = bclass->process (filter, rtp_buf);
-  if (out_buf) {
-    guint32 timestamp = gst_rtp_buffer_get_timestamp (rtp_buf);
-
-    /* push buffer with timestamp 
-     * We are assuming here that the timestamp of the last RTP buffer
-     * is the same as the timestamp wanted on the collector. If this is not a
-     * desired result, the process function should push itself with another
-     * timestamp and return NULL.
-     */
-    ret = gst_base_rtp_depayload_push_ts (filter, timestamp, out_buf);
-  }
-  gst_buffer_unref (rtp_buf);
-
-  return ret;
-}
-
 static void
 gst_base_rtp_depayload_set_gst_timestamp (GstBaseRTPDepayload * filter,
-    guint32 timestamp, GstBuffer * buf)
+    guint32 rtptime, GstBuffer * buf)
 {
-  GstClockTime ts, adjusted, exttimestamp;
   GstBaseRTPDepayloadPrivate *priv;
-  guint64 diff;
+  GstClockTime timestamp, duration;
 
   priv = filter->priv;
 
-  /* no clock-base set, take first timestamp as base */
-  if (priv->clock_base == -1)
-    priv->clock_base = timestamp;
+  timestamp = GST_BUFFER_TIMESTAMP (buf);
+  duration = GST_BUFFER_DURATION (buf);
 
-  if (priv->prev_timestamp == -1) {
-    priv->prev_timestamp = timestamp;
-    priv->ts_wraparound = 0;
-  }
+  /* apply last incomming timestamp and duration to outgoing buffer if
+   * not otherwise set. */
+  if (!GST_CLOCK_TIME_IS_VALID (timestamp))
+    GST_BUFFER_TIMESTAMP (buf) = priv->timestamp;
+  if (!GST_CLOCK_TIME_IS_VALID (duration))
+    GST_BUFFER_DURATION (buf) = priv->duration;
 
-  /* check for timestamp wraparound */
-  exttimestamp = timestamp + priv->ts_wraparound;
-
-  if (exttimestamp < priv->prev_timestamp)
-    diff = priv->prev_timestamp - exttimestamp;
-  else
-    diff = exttimestamp - priv->prev_timestamp;
-
-  if (diff > G_MAXINT32) {
-    /* timestamp went backwards more than allowed, we wrap around and get
-     * updated extended timestamp. */
-    priv->ts_wraparound += (G_GINT64_CONSTANT (1) << 32);
-    exttimestamp = timestamp + priv->ts_wraparound;
-  }
-  priv->prev_timestamp = exttimestamp;
-
-  /* rtp timestamps are based on the clock_rate
-   * gst timesamps are in nanoseconds */
-  ts = gst_util_uint64_scale_int (exttimestamp, GST_SECOND, filter->clock_rate);
-
-  GST_DEBUG_OBJECT (filter,
-      "timestamp: %u, wrap %" G_GUINT64_FORMAT ", clockrate : %u", timestamp,
-      priv->ts_wraparound, filter->clock_rate);
-
-  /* add delay to timestamp */
-  adjusted = ts + (filter->queue_delay * GST_MSECOND);
-
-  GST_DEBUG_OBJECT (filter, "RTP: %u, GST: %" GST_TIME_FORMAT ", adjusted %"
-      GST_TIME_FORMAT, timestamp, GST_TIME_ARGS (ts), GST_TIME_ARGS (adjusted));
-
-  GST_BUFFER_TIMESTAMP (buf) = adjusted;
-
-  /* if this is the first buf send a NEWSEGMENT */
+  /* if this is the first buffer send a NEWSEGMENT */
   if (filter->need_newsegment) {
     GstEvent *event;
-    GstClockTime start, stop, position;
-
-    start = gst_util_uint64_scale_int (priv->clock_base, GST_SECOND,
-        filter->clock_rate);
+    GstClockTime stop, position;
 
     if (priv->npt_stop != -1)
-      stop = priv->npt_stop - priv->npt_start + start;
+      stop = priv->npt_stop - priv->npt_start;
     else
       stop = -1;
 
@@ -545,7 +435,7 @@ gst_base_rtp_depayload_set_gst_timestamp (GstBaseRTPDepayload * filter,
 
     event =
         gst_event_new_new_segment_full (FALSE, priv->play_speed,
-        priv->play_scale, GST_FORMAT_TIME, start, stop, position);
+        priv->play_scale, GST_FORMAT_TIME, 0, stop, position);
 
     gst_pad_push_event (filter->srcpad, event);
 
@@ -554,148 +444,26 @@ gst_base_rtp_depayload_set_gst_timestamp (GstBaseRTPDepayload * filter,
   }
 }
 
-static void
-gst_base_rtp_depayload_queue_release (GstBaseRTPDepayload * filter)
-{
-  GQueue *queue = filter->queue;
-  guint32 headts, tailts;
-  GstBaseRTPDepayloadClass *bclass;
-  gfloat q_size_secs;
-  guint maxtsunits;
-
-  if (g_queue_is_empty (queue))
-    return;
-
-  /* if our queue is getting to big (more than RTP_QUEUEDELAY ms of data)
-   * release heading buffers
-   */
-  /*GST_DEBUG_OBJECT (filter, "clockrate %d, queue_delay %d", filter->clock_rate,
-     filter->queue_delay); */
-  q_size_secs = (gfloat) filter->queue_delay / 1000;
-  maxtsunits = (gfloat) filter->clock_rate * q_size_secs;
-
-  QUEUE_LOCK (filter);
-  headts =
-      gst_rtp_buffer_get_timestamp (GST_BUFFER (g_queue_peek_head (queue)));
-  tailts =
-      gst_rtp_buffer_get_timestamp (GST_BUFFER (g_queue_peek_tail (queue)));
-
-  bclass = GST_BASE_RTP_DEPAYLOAD_GET_CLASS (filter);
-
-  /*GST_DEBUG("maxtsunit is %u %u %u %u", maxtsunits, headts, tailts, headts - tailts); */
-  while (headts - tailts > maxtsunits) {
-    GST_DEBUG_OBJECT (filter, "Poping packet from queue");
-    if (bclass->process) {
-      GstBuffer *in = g_queue_pop_head (queue);
-
-      gst_base_rtp_depayload_process (filter, in);
-    }
-    headts =
-        gst_rtp_buffer_get_timestamp (GST_BUFFER (g_queue_peek_head (queue)));
-  }
-  QUEUE_UNLOCK (filter);
-}
-
-
-static gpointer
-gst_base_rtp_depayload_thread (GstBaseRTPDepayload * filter)
-{
-  while (filter->thread_running) {
-    gst_base_rtp_depayload_queue_release (filter);
-    /* sleep for 5msec (XXX: 5msec is a value that works for audio and video,
-     * should be adjusted based on frequency of incoming packet, 
-     * or by data comsumption rate of the sink (depends on how
-     * clock-drift compensation is implemented) */
-    gst_base_rtp_depayload_wait (filter, GST_MSECOND * 5);
-  }
-  return NULL;
-}
-
-static gboolean
-gst_base_rtp_depayload_start_thread (GstBaseRTPDepayload * filter)
-{
-  /* only launch the thread if processing is needed */
-  if (filter->queue_delay) {
-    GST_DEBUG_OBJECT (filter, "Starting queue release thread");
-    filter->thread_running = TRUE;
-    filter->thread =
-        g_thread_create ((GThreadFunc) gst_base_rtp_depayload_thread, filter,
-        TRUE, NULL);
-    GST_DEBUG_OBJECT (filter, "Started queue release thread");
-  }
-  return TRUE;
-}
-
-static gboolean
-gst_base_rtp_depayload_stop_thread (GstBaseRTPDepayload * filter)
-{
-  filter->thread_running = FALSE;
-
-  if (filter->thread) {
-    g_thread_join (filter->thread);
-    filter->thread = NULL;
-  }
-  QUEUE_LOCK_FREE (filter);
-  return TRUE;
-}
-
-static void
-gst_base_rtp_depayload_wait (GstBaseRTPDepayload * filter, GstClockTime time)
-{
-  GstClockID id;
-  GstClock *clock;
-  GstClockTime base;
-
-  g_return_if_fail (GST_CLOCK_TIME_IS_VALID (time));
-
-  GST_OBJECT_LOCK (filter);
-  if ((clock = GST_ELEMENT_CLOCK (filter)) == NULL)
-    goto no_clock;
-  gst_object_ref (clock);
-  GST_OBJECT_UNLOCK (filter);
-
-  base = gst_clock_get_time (clock);
-  id = gst_clock_new_single_shot_id (clock, base + time);
-
-  gst_object_unref (clock);
-
-  gst_clock_id_wait (id, NULL);
-  gst_clock_id_unref (id);
-
-  return;
-
-no_clock:
-  {
-    GST_DEBUG_OBJECT (filter, "No clock given yet");
-    GST_OBJECT_UNLOCK (filter);
-    return;
-  }
-}
-
 static GstStateChangeReturn
 gst_base_rtp_depayload_change_state (GstElement * element,
     GstStateChange transition)
 {
   GstBaseRTPDepayload *filter;
+  GstBaseRTPDepayloadPrivate *priv;
   GstStateChangeReturn ret;
 
   filter = GST_BASE_RTP_DEPAYLOAD (element);
-
-  /* we disallow changing the state from the thread */
-  if (g_thread_self () == filter->thread)
-    goto wrong_thread;
+  priv = filter->priv;
 
   switch (transition) {
     case GST_STATE_CHANGE_NULL_TO_READY:
-      if (!gst_base_rtp_depayload_start_thread (filter))
-        goto start_failed;
       break;
     case GST_STATE_CHANGE_READY_TO_PAUSED:
-      /* clock_rate needs to be overwritten by child */
-      filter->clock_rate = 0;
-      filter->priv->clock_base = -1;
-      filter->priv->ts_wraparound = 0;
       filter->need_newsegment = TRUE;
+      priv->npt_start = 0;
+      priv->npt_stop = -1;
+      priv->play_speed = 1.0;
+      priv->play_scale = 1.0;
       break;
     case GST_STATE_CHANGE_PAUSED_TO_PLAYING:
       break;
@@ -711,25 +479,11 @@ gst_base_rtp_depayload_change_state (GstElement * element,
     case GST_STATE_CHANGE_PAUSED_TO_READY:
       break;
     case GST_STATE_CHANGE_READY_TO_NULL:
-      gst_base_rtp_depayload_stop_thread (filter);
       break;
     default:
       break;
   }
   return ret;
-
-  /* ERRORS */
-wrong_thread:
-  {
-    GST_ELEMENT_ERROR (filter, CORE, STATE_CHANGE,
-        (NULL), ("cannot perform a state change from this thread"));
-    return GST_STATE_CHANGE_FAILURE;
-  }
-start_failed:
-  {
-    /* start method should have posted an error message */
-    return GST_STATE_CHANGE_FAILURE;
-  }
 }
 
 static void
