@@ -39,7 +39,7 @@
 #include "gstffmpegcolorspace.h"
 #include "gstffmpegcodecmap.h"
 
-GST_DEBUG_CATEGORY (ffmpegcolorspace_debug);
+GST_DEBUG_CATEGORY_STATIC (ffmpegcolorspace_debug);
 #define GST_CAT_DEFAULT ffmpegcolorspace_debug
 
 /* elementfactory information */
@@ -382,9 +382,10 @@ gst_ffmpegcsp_get_unit_size (GstBaseTransform * btrans, GstCaps * caps,
   GstFFMpegCsp *space = NULL;
   GstStructure *structure = NULL;
   AVCodecContext *ctx = NULL;
+  gboolean ret = TRUE;
   gint width, height;
 
-  g_return_val_if_fail (size, FALSE);
+  g_assert (size);
 
   space = GST_FFMPEGCSP (btrans);
 
@@ -396,8 +397,15 @@ gst_ffmpegcsp_get_unit_size (GstBaseTransform * btrans, GstCaps * caps,
 
   g_assert (ctx != NULL);
 
+  ctx->pix_fmt = PIX_FMT_NB;
+
   gst_ffmpegcsp_caps_with_codectype (CODEC_TYPE_VIDEO, caps, ctx);
 
+  if (G_UNLIKELY (ctx->pix_fmt == PIX_FMT_NB)) {
+    ret = FALSE;
+    goto beach;
+  }
+
   *size = avpicture_get_size (ctx->pix_fmt, width, height);
 
   /* ffmpeg frames have the palette after the frame data, whereas
@@ -408,11 +416,13 @@ gst_ffmpegcsp_get_unit_size (GstBaseTransform * btrans, GstCaps * caps,
     *size -= 4 * 256;           /* = AVPALETTE_SIZE */
   }
 
+beach:
+
   if (ctx->palctrl)
     av_free (ctx->palctrl);
   av_free (ctx);
 
-  return TRUE;
+  return ret;
 }
 
 #if 0
@@ -456,8 +466,7 @@ gst_ffmpegcsp_transform (GstBaseTransform * btrans, GstBuffer * inbuf,
   if (result == -1)
     goto not_supported;
 
-  /* copy timestamps */
-  gst_buffer_stamp (outbuf, inbuf);
+  /* baseclass copies timestamps */
   GST_DEBUG ("from %d -> to %d done", space->from_pixfmt, space->to_pixfmt);
 
   return GST_FLOW_OK;
