@@ -1,5 +1,5 @@
 /* GStreamer
- * Copyright (C) <1999> Erik Walthinsen <omega@cse.ogi.edu>
+ * Copyright (C) <2007> Wim Taymans <wim.taymans@gmail.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -1907,9 +1907,13 @@ analyse_source (GstPlayBaseBin * play_base_bin, gboolean * is_raw,
 
       templ = (GstPadTemplate *) walk->data;
       if (GST_PAD_TEMPLATE_DIRECTION (templ) == GST_PAD_SRC) {
-        if (GST_PAD_TEMPLATE_PRESENCE (templ) == GST_PAD_SOMETIMES)
+        if (GST_PAD_TEMPLATE_PRESENCE (templ) == GST_PAD_SOMETIMES) {
           *is_dynamic = TRUE;
-        break;
+          break;                /* only break out if we found a sometimes src pad 
+                                   continue walking through if say a request src pad is found
+                                   elements such as mpegtsparse and dvbbasebin have request
+                                   and sometimes src pads */
+        }
       }
       walk = g_list_next (walk);
     }
@@ -2380,6 +2384,17 @@ muted_group_change_state (GstElement * element,
 }
 #endif
 
+static void
+set_subtitles_visible (GstPlayBaseBin * play_base_bin, gboolean visible)
+{
+  GstPlayBaseBinClass *klass = GST_PLAY_BASE_BIN_GET_CLASS (play_base_bin);
+
+  /* we use a vfunc for this since we don't have a reference to the
+   * textoverlay element, but playbin does */
+  if (klass != NULL && klass->set_subtitles_visible != NULL)
+    klass->set_subtitles_visible (play_base_bin, visible);
+}
+
 /*
  * Caller has group-lock held.
  */
@@ -2403,6 +2418,17 @@ set_active_source (GstPlayBaseBin * play_base_bin,
     return;
   }
 
+  /* HACK: instead of unlinking the subtitle input (= lots of hassle,
+   * especially if subtitles come from an external source), just tell
+   * textoverlay not to render them */
+  if (type == GST_STREAM_TYPE_TEXT) {
+    gboolean visible = (source_num != -1);
+
+    set_subtitles_visible (play_base_bin, visible);
+    if (!visible)
+      return;
+  }
+
   sel = group->type[type - 1].selector;
 
   for (s = group->streaminfo; s; s = s->next) {
@@ -2421,8 +2447,7 @@ set_active_source (GstPlayBaseBin * play_base_bin,
                 "pb_sel_pad"));
 
         if (sel && sel_pad != NULL) {
-          g_object_set (G_OBJECT (sel), "active-pad", GST_PAD_NAME (sel_pad),
-              NULL);
+          g_object_set (G_OBJECT (sel), "active-pad", sel_pad, NULL);
         }
 
         have_active = TRUE;
@@ -2441,7 +2466,7 @@ set_active_source (GstPlayBaseBin * play_base_bin,
 
   if (!have_active) {
     GST_LOG ("Muting group type: %d", type);
-    g_object_set (sel, "active-pad", "", NULL);
+    g_object_set (sel, "active-pad", NULL, NULL);
   } else {
     GST_LOG ("Unmuting group type: %d", type);
   }
