@@ -249,8 +249,10 @@
 #include <gst/pbutils/pbutils.h>
 
 #include "gstplay-enum.h"
+#include "gstplay-marshal.h"
 #include "gstplaysink.h"
 #include "gstfactorylists.h"
+#include "gstscreenshot.h"
 #include "gststreaminfo.h"
 #include "gststreamselector.h"
 
@@ -277,7 +279,7 @@ struct _GstSourceSelect
   GstPlaySinkType type;         /* the sink pad type of the selector */
 
   GstElement *selector;         /* the selector */
-  gint current;                 /* the currently selected stream */
+  GPtrArray *channels;
   GstPad *srcpad;               /* the source pad of the selector */
   GstPad *sinkpad;              /* the sinkpad of the sink when the selector is linked */
 };
@@ -298,6 +300,10 @@ struct _GstSourceGroup
   GstElement *source;
   gchar *subencoding;           /* encoding to propagate to the subtitle elements */
 
+  GPtrArray *video_channels;    /* links to selector pads */
+  GPtrArray *audio_channels;    /* links to selector pads */
+  GPtrArray *text_channels;     /* links to selector pads */
+
   /* uridecodebins for uri and subtitle uri */
   GstElement *uridecodebin;
   GstElement *suburidecodebin;
@@ -319,6 +325,9 @@ struct _GstPlayBin
 
   /* properties */
   guint connection_speed;       /* connection speed in bits/sec (0 = unknown) */
+  gint current_video;           /* the currently selected stream */
+  gint current_audio;           /* the currently selected stream */
+  gint current_text;            /* the currently selected stream */
 
   /* our play sink */
   GstPlaySink *playsink;
@@ -330,13 +339,27 @@ struct _GstPlayBinClass
 {
   GstPipelineClass parent_class;
 
-  void (*about_to_finish) (void);
+  /* notify app that the current uri finished decoding and it is possible to
+   * queue a new one for gapless playback */
+  void (*about_to_finish) (GstPlayBin * playbin);
+
+  /* notify app that number of audio/video/text streams changed */
+  void (*video_changed) (GstPlayBin * playbin);
+  void (*audio_changed) (GstPlayBin * playbin);
+  void (*text_changed) (GstPlayBin * playbin);
+
+  /* get audio/video/text tags for a stream */
+  GstTagList *(*get_video_tags) (GstPlayBin * playbin, gint stream);
+  GstTagList *(*get_audio_tags) (GstPlayBin * playbin, gint stream);
+  GstTagList *(*get_text_tags) (GstPlayBin * playbin, gint stream);
+
+  /* get the last video frame and convert it to the given caps */
+  GstBuffer *(*convert_frame) (GstPlayBin * playbin, GstCaps * caps);
 };
 
 /* props */
 #define DEFAULT_URI               NULL
 #define DEFAULT_SUBURI            NULL
-#define DEFAULT_STREAMINFO        NULL
 #define DEFAULT_SOURCE            NULL
 #define DEFAULT_FLAGS             GST_PLAY_FLAG_AUDIO | GST_PLAY_FLAG_VIDEO | GST_PLAY_FLAG_TEXT | \
 	                          GST_PLAY_FLAG_SOFT_VOLUME
@@ -351,6 +374,7 @@ struct _GstPlayBinClass
 #define DEFAULT_VIDEO_SINK        NULL
 #define DEFAULT_VIS_PLUGIN        NULL
 #define DEFAULT_VOLUME            1.0
+#define DEFAULT_MUTE              FALSE
 #define DEFAULT_FRAME             NULL
 #define DEFAULT_FONT_DESC         NULL
 #define DEFAULT_CONNECTION_SPEED  0
@@ -360,7 +384,6 @@ enum
   PROP_0,
   PROP_URI,
   PROP_SUBURI,
-  PROP_STREAMINFO,
   PROP_SOURCE,
   PROP_FLAGS,
   PROP_N_VIDEO,
@@ -374,6 +397,7 @@ enum
   PROP_VIDEO_SINK,
   PROP_VIS_PLUGIN,
   PROP_VOLUME,
+  PROP_MUTE,
   PROP_FRAME,
   PROP_FONT_DESC,
   PROP_CONNECTION_SPEED
@@ -383,6 +407,12 @@ enum
 enum
 {
   SIGNAL_ABOUT_TO_FINISH,
+  SIGNAL_VIDEO_CHANGED,
+  SIGNAL_AUDIO_CHANGED,
+  SIGNAL_TEXT_CHANGED,
+  SIGNAL_GET_VIDEO_TAGS,
+  SIGNAL_GET_AUDIO_TAGS,
+  SIGNAL_GET_TEXT_TAGS,
   LAST_SIGNAL
 };
 
@@ -400,6 +430,16 @@ static GstStateChangeReturn gst_play_bin_change_state (GstElement * element,
 
 static void gst_play_bin_handle_message (GstBin * bin, GstMessage * message);
 
+static GstTagList *gst_play_bin_get_video_tags (GstPlayBin * playbin,
+    gint stream);
+static GstTagList *gst_play_bin_get_audio_tags (GstPlayBin * playbin,
+    gint stream);
+static GstTagList *gst_play_bin_get_text_tags (GstPlayBin * playbin,
+    gint stream);
+
+static GstBuffer *gst_play_bin_convert_frame (GstPlayBin * playbin,
+    GstCaps * caps);
+
 static gboolean setup_next_source (GstPlayBin * playbin);
 
 static GstElementClass *parent_class;
@@ -412,6 +452,38 @@ GST_ELEMENT_DETAILS ("Player Bin 2",
     "Autoplug and play media from an uri",
     "Wim Taymans <wim.taymans@gmail.com>");
 
+static void
+gst_play_marshal_BUFFER__BOXED (GClosure * closure,
+    GValue * return_value G_GNUC_UNUSED,
+    guint n_param_values,
+    const GValue * param_values,
+    gpointer invocation_hint G_GNUC_UNUSED, gpointer marshal_data)
+{
+  typedef GstBuffer *(*GMarshalFunc_OBJECT__BOXED) (gpointer data1,
+      gpointer arg_1, gpointer data2);
+  register GMarshalFunc_OBJECT__BOXED callback;
+  register GCClosure *cc = (GCClosure *) closure;
+  register gpointer data1, data2;
+  GstBuffer *v_return;
+
+  g_return_if_fail (return_value != NULL);
+  g_return_if_fail (n_param_values == 2);
+
+  if (G_CCLOSURE_SWAP_DATA (closure)) {
+    data1 = closure->data;
+    data2 = g_value_peek_pointer (param_values + 0);
+  } else {
+    data1 = g_value_peek_pointer (param_values + 0);
+    data2 = closure->data;
+  }
+  callback =
+      (GMarshalFunc_OBJECT__BOXED) (marshal_data ? marshal_data : cc->callback);
+
+  v_return = callback (data1, g_value_get_boxed (param_values + 1), data2);
+
+  gst_value_take_buffer (return_value, v_return);
+}
+
 static GType
 gst_play_bin_get_type (void)
 {
@@ -476,16 +548,16 @@ gst_play_bin_class_init (GstPlayBinClass * klass)
       g_param_spec_string ("suburi", ".sub-URI", "Optional URI of a subtitle",
           NULL, G_PARAM_READWRITE));
 
-  g_object_class_install_property (gobject_klass, PROP_STREAMINFO,
-      g_param_spec_value_array ("stream-info",
-          "StreamInfo GValueArray", "value array of streaminfo",
-          g_param_spec_object ("streaminfo", "StreamInfo", "Streaminfo object",
-              GST_TYPE_STREAM_INFO, G_PARAM_READABLE), G_PARAM_READABLE));
-
   g_object_class_install_property (gobject_klass, PROP_SOURCE,
       g_param_spec_object ("source", "Source", "Source element",
           GST_TYPE_ELEMENT, G_PARAM_READABLE));
 
+
+  /**
+   * GstPlayBin:flags
+   *
+   * Control the behaviour of playbin.
+   */
   g_object_class_install_property (gobject_klass, PROP_FLAGS,
       g_param_spec_flags ("flags", "Flags", "Flags to control behaviour",
           GST_TYPE_PLAY_FLAGS, DEFAULT_FLAGS, G_PARAM_READWRITE));
@@ -503,13 +575,11 @@ gst_play_bin_class_init (GstPlayBinClass * klass)
    *
    * Get or set the currently playing video stream. By default the first video
    * stream with data is played.
-   *
-   * Setting this property to -2 will disable the video stream.
    */
   g_object_class_install_property (gobject_klass, PROP_CURRENT_VIDEO,
       g_param_spec_int ("current-video", "Current Video",
-          "Currently playing video stream (-1 = auto, -2 = none)",
-          -2, G_MAXINT, -1, G_PARAM_READWRITE));
+          "Currently playing video stream (-1 = auto)",
+          -1, G_MAXINT, -1, G_PARAM_READWRITE));
   /**
    * GstPlayBin:n-audio
    *
@@ -523,13 +593,11 @@ gst_play_bin_class_init (GstPlayBinClass * klass)
    *
    * Get or set the currently playing audio stream. By default the first audio
    * stream with data is played.
-   *
-   * Setting this property to -2 will disable the audio stream.
    */
   g_object_class_install_property (gobject_klass, PROP_CURRENT_AUDIO,
       g_param_spec_int ("current-audio", "Current audio",
-          "Currently playing audio stream (-1 = none, -2 = none)",
-          -2, G_MAXINT, -1, G_PARAM_READWRITE));
+          "Currently playing audio stream (-1 = auto)",
+          -1, G_MAXINT, -1, G_PARAM_READWRITE));
   /**
    * GstPlayBin:n-text
    *
@@ -541,15 +609,13 @@ gst_play_bin_class_init (GstPlayBinClass * klass)
   /**
    * GstPlayBin:current-text
    *
-   * Get or set the currently playing subtitle stream. By default the first audio
-   * stream with data is played.
-   *
-   * Setting this property to -2 will disable the text stream.
+   * Get or set the currently playing subtitle stream. By default the first
+   * subtitle stream with data is played.
    */
   g_object_class_install_property (gobject_klass, PROP_CURRENT_TEXT,
       g_param_spec_int ("current-text", "Current Text",
-          "Currently playing text stream (-1 = none, -2 = none)",
-          -2, G_MAXINT, -1, G_PARAM_READWRITE));
+          "Currently playing text stream (-1 = auto)",
+          -1, G_MAXINT, -1, G_PARAM_READWRITE));
 
   g_object_class_install_property (gobject_klass, PROP_SUBTITLE_ENCODING,
       g_param_spec_string ("subtitle-encoding", "subtitle encoding",
@@ -572,8 +638,20 @@ gst_play_bin_class_init (GstPlayBinClass * klass)
           GST_TYPE_ELEMENT, G_PARAM_READWRITE));
 
   g_object_class_install_property (gobject_klass, PROP_VOLUME,
-      g_param_spec_double ("volume", "volume", "volume",
+      g_param_spec_double ("volume", "Volume", "The audio volume",
           0.0, VOLUME_MAX_DOUBLE, 1.0, G_PARAM_READWRITE));
+  g_object_class_install_property (gobject_klass, PROP_MUTE,
+      g_param_spec_boolean ("mute", "Mute",
+          "Mute the audio channel without changing the volume", FALSE,
+          G_PARAM_READWRITE));
+
+  /**
+   * GstPlayBin::frame
+   * @playbin: a #GstPlayBin
+   *
+   * Get the currently rendered or prerolled frame in the sink.
+   * The #GstCaps on the buffer will describe the format of the buffer.
+   */
   g_object_class_install_property (gobject_klass, PROP_FRAME,
       gst_param_spec_mini_object ("frame", "Frame",
           "The last frame (NULL = no video available)",
@@ -590,6 +668,7 @@ gst_play_bin_class_init (GstPlayBinClass * klass)
           0, G_MAXUINT, DEFAULT_CONNECTION_SPEED, G_PARAM_READWRITE));
   /**
    * GstPlayBin::about-to-finish:
+   * @playbin: a #GstPlayBin
    *
    * This signal is emitted when the current uri is about to finish. You can
    * set the next-uri and next-suburi to make sure that playback continues.
@@ -600,6 +679,121 @@ gst_play_bin_class_init (GstPlayBinClass * klass)
       G_STRUCT_OFFSET (GstPlayBinClass, about_to_finish), NULL, NULL,
       gst_marshal_VOID__VOID, G_TYPE_NONE, 0, G_TYPE_NONE);
 
+  /**
+   * GstPlayBin::video-changed
+   * @playbin: a #GstPlayBin
+   *
+   * This signal is emited whenever the number or order of the video
+   * streams has changed. The application will most likely want to select
+   * a new video stream.
+   */
+  gst_play_bin_signals[SIGNAL_VIDEO_CHANGED] =
+      g_signal_new ("video-changed", G_TYPE_FROM_CLASS (klass),
+      G_SIGNAL_RUN_LAST,
+      G_STRUCT_OFFSET (GstPlayBinClass, video_changed), NULL, NULL,
+      gst_marshal_VOID__VOID, G_TYPE_NONE, 0, G_TYPE_NONE);
+  /**
+   * GstPlayBin::audio-changed
+   * @playbin: a #GstPlayBin
+   *
+   * This signal is emited whenever the number or order of the audio
+   * streams has changed. The application will most likely want to select
+   * a new audio stream.
+   */
+  gst_play_bin_signals[SIGNAL_AUDIO_CHANGED] =
+      g_signal_new ("audio-changed", G_TYPE_FROM_CLASS (klass),
+      G_SIGNAL_RUN_LAST,
+      G_STRUCT_OFFSET (GstPlayBinClass, audio_changed), NULL, NULL,
+      gst_marshal_VOID__VOID, G_TYPE_NONE, 0, G_TYPE_NONE);
+  /**
+   * GstPlayBin::text-changed
+   * @playbin: a #GstPlayBin
+   *
+   * This signal is emited whenever the number or order of the text
+   * streams has changed. The application will most likely want to select
+   * a new text stream.
+   */
+  gst_play_bin_signals[SIGNAL_TEXT_CHANGED] =
+      g_signal_new ("text-changed", G_TYPE_FROM_CLASS (klass),
+      G_SIGNAL_RUN_LAST,
+      G_STRUCT_OFFSET (GstPlayBinClass, text_changed), NULL, NULL,
+      gst_marshal_VOID__VOID, G_TYPE_NONE, 0, G_TYPE_NONE);
+
+  /**
+   * GstPlayBin::get-video-tags
+   * @playbin: a #GstPlayBin
+   * @stream: a video stream number
+   *
+   * Action signal to retrieve the tags of a specific video stream number.
+   * This information can be used to select a stream.
+   *
+   * Returns: a GstTagList with tags or NULL when the stream number does not
+   * exist.
+   */
+  gst_play_bin_signals[SIGNAL_GET_VIDEO_TAGS] =
+      g_signal_new ("get-video-tags", G_TYPE_FROM_CLASS (klass),
+      G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
+      G_STRUCT_OFFSET (GstPlayBinClass, get_video_tags), NULL, NULL,
+      gst_play_marshal_BOXED__INT, GST_TYPE_TAG_LIST, 1, G_TYPE_INT);
+  /**
+   * GstPlayBin::get-audio-tags
+   * @playbin: a #GstPlayBin
+   * @stream: an audio stream number
+   *
+   * Action signal to retrieve the tags of a specific audio stream number.
+   * This information can be used to select a stream.
+   *
+   * Returns: a GstTagList with tags or NULL when the stream number does not
+   * exist.
+   */
+  gst_play_bin_signals[SIGNAL_GET_AUDIO_TAGS] =
+      g_signal_new ("get-audio-tags", G_TYPE_FROM_CLASS (klass),
+      G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
+      G_STRUCT_OFFSET (GstPlayBinClass, get_audio_tags), NULL, NULL,
+      gst_play_marshal_BOXED__INT, GST_TYPE_TAG_LIST, 1, G_TYPE_INT);
+  /**
+   * GstPlayBin::get-text-tags
+   * @playbin: a #GstPlayBin
+   * @stream: a text stream number
+   *
+   * Action signal to retrieve the tags of a specific text stream number.
+   * This information can be used to select a stream.
+   *
+   * Returns: a GstTagList with tags or NULL when the stream number does not
+   * exist.
+   */
+  gst_play_bin_signals[SIGNAL_GET_TEXT_TAGS] =
+      g_signal_new ("get-text-tags", G_TYPE_FROM_CLASS (klass),
+      G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
+      G_STRUCT_OFFSET (GstPlayBinClass, get_text_tags), NULL, NULL,
+      gst_play_marshal_BOXED__INT, GST_TYPE_TAG_LIST, 1, G_TYPE_INT);
+  /**
+   * GstPlayBin::convert-frame
+   * @playbin: a #GstPlayBin
+   * @caps: the target format of the frame
+   *
+   * Action signal to retrieve the currently playing video frame in the format
+   * specified by @caps.
+   * If @caps is %NULL, no conversion will be performed and this function is
+   * equivalent to the #GstPlayBin::frame property.
+   *
+   * Returns: a #GstBuffer of the current video frame converted to #caps. 
+   * The caps on the buffer will describe the final layout of the buffer data.
+   * %NULL is returned when no current buffer can be retrieved or when the
+   * conversion failed.
+   */
+  gst_play_bin_signals[SIGNAL_GET_TEXT_TAGS] =
+      g_signal_new ("convert-frame", G_TYPE_FROM_CLASS (klass),
+      G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
+      G_STRUCT_OFFSET (GstPlayBinClass, convert_frame), NULL, NULL,
+      gst_play_marshal_BUFFER__BOXED, GST_TYPE_BUFFER, 1, GST_TYPE_CAPS);
+
+  klass->get_video_tags = gst_play_bin_get_video_tags;
+  klass->get_audio_tags = gst_play_bin_get_audio_tags;
+  klass->get_text_tags = gst_play_bin_get_text_tags;
+
+  klass->convert_frame = gst_play_bin_convert_frame;
+
   gst_element_class_set_details (gstelement_klass, &gst_play_bin_details);
 
   gstelement_klass->change_state =
@@ -612,18 +806,27 @@ gst_play_bin_class_init (GstPlayBinClass * klass)
 static void
 init_group (GstPlayBin * playbin, GstSourceGroup * group)
 {
+  /* store the array for the different channels */
+  group->video_channels = g_ptr_array_new ();
+  group->audio_channels = g_ptr_array_new ();
+  group->text_channels = g_ptr_array_new ();
   /* init selectors */
   group->playbin = playbin;
   group->selector[0].media = "audio/x-raw-";
   group->selector[0].type = GST_PLAY_SINK_TYPE_AUDIO_RAW;
+  group->selector[0].channels = group->audio_channels;
   group->selector[1].media = "audio/";
   group->selector[1].type = GST_PLAY_SINK_TYPE_AUDIO;
+  group->selector[1].channels = group->audio_channels;
   group->selector[2].media = "video/x-raw-";
   group->selector[2].type = GST_PLAY_SINK_TYPE_VIDEO_RAW;
+  group->selector[2].channels = group->video_channels;
   group->selector[3].media = "video/";
   group->selector[3].type = GST_PLAY_SINK_TYPE_VIDEO;
+  group->selector[3].channels = group->video_channels;
   group->selector[4].media = "text/";
   group->selector[4].type = GST_PLAY_SINK_TYPE_TEXT;
+  group->selector[4].channels = group->text_channels;
 }
 
 static void
@@ -647,7 +850,9 @@ gst_play_bin_init (GstPlayBin * playbin)
   gst_bin_add (GST_BIN_CAST (playbin), GST_ELEMENT_CAST (playbin->playsink));
   gst_play_sink_set_flags (playbin->playsink, DEFAULT_FLAGS);
 
-  /* get the caps */
+  playbin->current_video = DEFAULT_CURRENT_VIDEO;
+  playbin->current_audio = DEFAULT_CURRENT_AUDIO;
+  playbin->current_text = DEFAULT_CURRENT_TEXT;
 }
 
 static void
@@ -706,6 +911,218 @@ done:
   GST_OBJECT_UNLOCK (playbin);
 }
 
+/* get the currently playing group or if nothing is playing, the next
+ * group. Must be called with the LOCK. */
+static GstSourceGroup *
+get_group (GstPlayBin * playbin)
+{
+  GstSourceGroup *result;
+
+  if (!(result = playbin->curr_group))
+    result = playbin->next_group;
+
+  return result;
+}
+
+static GstTagList *
+get_tags (GstPlayBin * playbin, GPtrArray * channels, gint stream)
+{
+  GstTagList *result;
+  GstPad *sinkpad;
+
+  if (!channels || channels->len < stream)
+    return NULL;
+
+  sinkpad = g_ptr_array_index (channels, stream);
+  g_object_get (sinkpad, "tags", &result, NULL);
+
+  return result;
+}
+
+static GstTagList *
+gst_play_bin_get_video_tags (GstPlayBin * playbin, gint stream)
+{
+  GstTagList *result;
+  GstSourceGroup *group;
+
+  GST_OBJECT_LOCK (playbin);
+  group = get_group (playbin);
+  result = get_tags (playbin, group->video_channels, stream);
+  GST_OBJECT_UNLOCK (playbin);
+
+  return result;
+}
+
+static GstTagList *
+gst_play_bin_get_audio_tags (GstPlayBin * playbin, gint stream)
+{
+  GstTagList *result;
+  GstSourceGroup *group;
+
+  GST_OBJECT_LOCK (playbin);
+  group = get_group (playbin);
+  result = get_tags (playbin, group->audio_channels, stream);
+  GST_OBJECT_UNLOCK (playbin);
+
+  return result;
+}
+
+static GstTagList *
+gst_play_bin_get_text_tags (GstPlayBin * playbin, gint stream)
+{
+  GstTagList *result;
+  GstSourceGroup *group;
+
+  GST_OBJECT_LOCK (playbin);
+  group = get_group (playbin);
+  result = get_tags (playbin, group->text_channels, stream);
+  GST_OBJECT_UNLOCK (playbin);
+
+  return result;
+}
+
+static GstBuffer *
+gst_play_bin_convert_frame (GstPlayBin * playbin, GstCaps * caps)
+{
+  GstBuffer *result;
+
+  result = gst_play_sink_get_last_frame (playbin->playsink);
+  if (result != NULL && caps != NULL) {
+    GstBuffer *temp;
+
+    temp = gst_play_frame_conv_convert (result, caps);
+    gst_buffer_unref (result);
+    result = temp;
+  }
+  return result;
+}
+
+static gboolean
+gst_play_bin_set_current_video_stream (GstPlayBin * playbin, gint stream)
+{
+  GstSourceGroup *group;
+  GPtrArray *channels;
+  GstPad *sinkpad;
+
+  GST_OBJECT_LOCK (playbin);
+  group = get_group (playbin);
+  if (!(channels = group->video_channels))
+    goto no_channels;
+
+  if (stream == -1 || channels->len < stream) {
+    sinkpad = NULL;
+  } else {
+    /* take channel from selected stream */
+    sinkpad = g_ptr_array_index (channels, stream);
+  }
+
+  if (sinkpad)
+    gst_object_ref (sinkpad);
+  GST_OBJECT_UNLOCK (playbin);
+
+  if (sinkpad) {
+    GstObject *selector;
+
+    if ((selector = gst_pad_get_parent (sinkpad))) {
+      /* activate the selected pad */
+      g_object_set (selector, "active-pad", sinkpad, NULL);
+      gst_object_unref (selector);
+    }
+    gst_object_unref (sinkpad);
+  }
+  return TRUE;
+
+no_channels:
+  {
+    GST_OBJECT_UNLOCK (playbin);
+    return FALSE;
+  }
+}
+
+static gboolean
+gst_play_bin_set_current_audio_stream (GstPlayBin * playbin, gint stream)
+{
+  GstSourceGroup *group;
+  GPtrArray *channels;
+  GstPad *sinkpad;
+
+  GST_OBJECT_LOCK (playbin);
+  group = get_group (playbin);
+  if (!(channels = group->audio_channels))
+    goto no_channels;
+
+  if (stream == -1 || channels->len < stream) {
+    sinkpad = NULL;
+  } else {
+    /* take channel from selected stream */
+    sinkpad = g_ptr_array_index (channels, stream);
+  }
+
+  if (sinkpad)
+    gst_object_ref (sinkpad);
+  GST_OBJECT_UNLOCK (playbin);
+
+  if (sinkpad) {
+    GstObject *selector;
+
+    if ((selector = gst_pad_get_parent (sinkpad))) {
+      /* activate the selected pad */
+      g_object_set (selector, "active-pad", sinkpad, NULL);
+      gst_object_unref (selector);
+    }
+    gst_object_unref (sinkpad);
+  }
+  return TRUE;
+
+no_channels:
+  {
+    GST_OBJECT_UNLOCK (playbin);
+    return FALSE;
+  }
+}
+
+static gboolean
+gst_play_bin_set_current_text_stream (GstPlayBin * playbin, gint stream)
+{
+  GstSourceGroup *group;
+  GPtrArray *channels;
+  GstPad *sinkpad;
+
+  GST_OBJECT_LOCK (playbin);
+  group = get_group (playbin);
+  if (!(channels = group->text_channels))
+    goto no_channels;
+
+  if (stream == -1 || channels->len < stream) {
+    sinkpad = NULL;
+  } else {
+    /* take channel from selected stream */
+    sinkpad = g_ptr_array_index (channels, stream);
+  }
+
+  if (sinkpad)
+    gst_object_ref (sinkpad);
+  GST_OBJECT_UNLOCK (playbin);
+
+  if (sinkpad) {
+    GstObject *selector;
+
+    if ((selector = gst_pad_get_parent (sinkpad))) {
+      /* activate the selected pad */
+      g_object_set (selector, "active-pad", sinkpad, NULL);
+      gst_object_unref (selector);
+    }
+    gst_object_unref (sinkpad);
+  }
+  return TRUE;
+
+no_channels:
+  {
+    GST_OBJECT_UNLOCK (playbin);
+    return FALSE;
+  }
+}
+
 static void
 gst_play_bin_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec)
@@ -725,10 +1142,13 @@ gst_play_bin_set_property (GObject * object, guint prop_id,
       gst_play_sink_set_flags (playbin->playsink, g_value_get_flags (value));
       break;
     case PROP_CURRENT_VIDEO:
+      gst_play_bin_set_current_video_stream (playbin, g_value_get_int (value));
       break;
     case PROP_CURRENT_AUDIO:
+      gst_play_bin_set_current_audio_stream (playbin, g_value_get_int (value));
       break;
     case PROP_CURRENT_TEXT:
+      gst_play_bin_set_current_text_stream (playbin, g_value_get_int (value));
       break;
     case PROP_SUBTITLE_ENCODING:
       break;
@@ -737,8 +1157,14 @@ gst_play_bin_set_property (GObject * object, guint prop_id,
     case PROP_AUDIO_SINK:
       break;
     case PROP_VIS_PLUGIN:
+      gst_play_sink_set_vis_plugin (playbin->playsink,
+          g_value_get_object (value));
       break;
     case PROP_VOLUME:
+      gst_play_sink_set_volume (playbin->playsink, g_value_get_double (value));
+      break;
+    case PROP_MUTE:
+      gst_play_sink_set_mute (playbin->playsink, g_value_get_boolean (value));
       break;
     case PROP_FONT_DESC:
       break;
@@ -763,42 +1189,80 @@ gst_play_bin_get_property (GObject * object, guint prop_id, GValue * value,
 
   switch (prop_id) {
     case PROP_URI:
+    {
+      GstSourceGroup *group;
+
       GST_OBJECT_LOCK (playbin);
-      /* get the currently playing group first, then the queued one, just in
-       * case we did not yet start playback. */
-      if (playbin->curr_group)
-        g_value_set_string (value, playbin->curr_group->uri);
-      else
-        g_value_set_string (value, playbin->next_group->uri);
+      group = get_group (playbin);
+      g_value_set_string (value, group->uri);
       GST_OBJECT_UNLOCK (playbin);
       break;
+    }
     case PROP_SUBURI:
+    {
+      GstSourceGroup *group;
+
       GST_OBJECT_LOCK (playbin);
-      /* get the currently playing group first, then the queued one */
-      if (playbin->curr_group)
-        g_value_set_string (value, playbin->curr_group->suburi);
-      else
-        g_value_set_string (value, playbin->next_group->suburi);
+      group = get_group (playbin);
+      g_value_set_string (value, group->suburi);
       GST_OBJECT_UNLOCK (playbin);
       break;
-    case PROP_STREAMINFO:
-      break;
+    }
     case PROP_SOURCE:
       break;
     case PROP_FLAGS:
       g_value_set_flags (value, gst_play_sink_get_flags (playbin->playsink));
       break;
     case PROP_N_VIDEO:
+    {
+      GstSourceGroup *group;
+      gint n_video;
+
+      GST_OBJECT_LOCK (playbin);
+      group = get_group (playbin);
+      n_video = (group->video_channels ? group->video_channels->len : 0);
+      g_value_set_int (value, n_video);
+      GST_OBJECT_UNLOCK (playbin);
       break;
+    }
     case PROP_CURRENT_VIDEO:
+      GST_OBJECT_LOCK (playbin);
+      g_value_set_int (value, playbin->current_video);
+      GST_OBJECT_UNLOCK (playbin);
       break;
     case PROP_N_AUDIO:
+    {
+      GstSourceGroup *group;
+      gint n_audio;
+
+      GST_OBJECT_LOCK (playbin);
+      group = get_group (playbin);
+      n_audio = (group->audio_channels ? group->audio_channels->len : 0);
+      g_value_set_int (value, n_audio);
+      GST_OBJECT_UNLOCK (playbin);
       break;
+    }
     case PROP_CURRENT_AUDIO:
+      GST_OBJECT_LOCK (playbin);
+      g_value_set_int (value, playbin->current_audio);
+      GST_OBJECT_UNLOCK (playbin);
       break;
     case PROP_N_TEXT:
+    {
+      GstSourceGroup *group;
+      gint n_text;
+
+      GST_OBJECT_LOCK (playbin);
+      group = get_group (playbin);
+      n_text = (group->text_channels ? group->text_channels->len : 0);
+      g_value_set_int (value, n_text);
+      GST_OBJECT_UNLOCK (playbin);
       break;
+    }
     case PROP_CURRENT_TEXT:
+      GST_OBJECT_LOCK (playbin);
+      g_value_set_int (value, playbin->current_text);
+      GST_OBJECT_UNLOCK (playbin);
       break;
     case PROP_SUBTITLE_ENCODING:
       break;
@@ -809,8 +1273,13 @@ gst_play_bin_get_property (GObject * object, guint prop_id, GValue * value,
     case PROP_VIS_PLUGIN:
       break;
     case PROP_VOLUME:
+      g_value_set_double (value, gst_play_sink_get_volume (playbin->playsink));
+      break;
+    case PROP_MUTE:
+      g_value_set_boolean (value, gst_play_sink_get_mute (playbin->playsink));
       break;
     case PROP_FRAME:
+      gst_value_take_buffer (value, gst_play_bin_convert_frame (playbin, NULL));
       break;
     case PROP_FONT_DESC:
       break;
@@ -878,7 +1347,7 @@ pad_added_cb (GstElement * decodebin, GstPad * pad, GstSourceGroup * group)
       GST_DEBUG_PAD_NAME (pad), caps, group);
 
   /* major type of the pad, this determines the selector to use */
-  for (i = 0; i < 3; i++) {
+  for (i = 0; i < GST_PLAY_SINK_TYPE_LAST; i++) {
     if (g_str_has_prefix (name, group->selector[i].media)) {
       select = &group->selector[i];
       break;
@@ -908,12 +1377,20 @@ pad_added_cb (GstElement * decodebin, GstPad * pad, GstSourceGroup * group)
   if ((sinkpad = gst_element_get_request_pad (select->selector, "sink%d"))) {
     GST_DEBUG_OBJECT (playbin, "got pad %s:%s from selector",
         GST_DEBUG_PAD_NAME (sinkpad));
+
+    /* store the selector for the pad */
+    g_object_set_data (G_OBJECT (sinkpad), "playbin2.select", select);
+
+    /* store the pad in the array */
+    GST_DEBUG_OBJECT (playbin, "pad %p added to array", sinkpad);
+    g_ptr_array_add (select->channels, sinkpad);
+
     res = gst_pad_link (pad, sinkpad);
     if (GST_PAD_LINK_FAILED (res))
       goto link_failed;
 
     /* store selector pad so we can release it */
-    g_object_set_data (G_OBJECT (pad), "playbin2.selector", sinkpad);
+    g_object_set_data (G_OBJECT (pad), "playbin2.sinkpad", sinkpad);
   }
   GST_DEBUG_OBJECT (playbin, "linked pad %s:%s to selector %p",
       GST_DEBUG_PAD_NAME (pad), select->selector);
@@ -950,6 +1427,7 @@ pad_removed_cb (GstElement * decodebin, GstPad * pad, GstSourceGroup * group)
   GstPlayBin *playbin;
   GstPad *peer;
   GstElement *selector;
+  GstSourceSelect *select;
 
   playbin = group->playbin;
 
@@ -957,10 +1435,15 @@ pad_removed_cb (GstElement * decodebin, GstPad * pad, GstSourceGroup * group)
       "pad %s:%s removed from group %p", GST_DEBUG_PAD_NAME (pad), group);
 
   /* get the selector sinkpad */
-  peer = g_object_get_data (G_OBJECT (pad), "playbin2.selector");
-  if (!peer)
+  if (!(peer = g_object_get_data (G_OBJECT (pad), "playbin2.sinkpad")))
     goto not_linked;
 
+  if ((select = g_object_get_data (G_OBJECT (peer), "playbin2.select"))) {
+    /* remove the pad from the array */
+    g_ptr_array_remove (select->channels, peer);
+    GST_DEBUG_OBJECT (playbin, "pad %p removed from array", peer);
+  }
+
   /* unlink the pad now (can fail, the pad is unlinked before it's removed) */
   gst_pad_unlink (pad, peer);
 
@@ -1011,14 +1494,15 @@ no_more_pads_cb (GstElement * decodebin, GstSourceGroup * group)
 
   GST_DEBUG_OBJECT (playbin, "no more pads in group %p", group);
 
-  for (i = 0; i < 3; i++) {
+  for (i = 0; i < GST_PLAY_SINK_TYPE_LAST; i++) {
     GstSourceSelect *select = &group->selector[i];
 
     if (select->selector) {
       select->sinkpad =
           gst_play_sink_request_pad (playbin->playsink, select->type);
       res = gst_pad_link (select->srcpad, select->sinkpad);
-      GST_DEBUG_OBJECT (playbin, "linked type %s: %d", select->media, res);
+      GST_DEBUG_OBJECT (playbin, "linked type %s, result: %d", select->media,
+          res);
     }
   }
   /* configure the modes now */
@@ -1035,7 +1519,7 @@ perform_eos (GstPlayBin * playbin, GstSourceGroup * group)
   GST_DEBUG_OBJECT (playbin, "doing EOS in group %p", group);
 
   event = gst_event_new_eos ();
-  for (i = 0; i < 3; i++) {
+  for (i = 0; i < GST_PLAY_SINK_TYPE_LAST; i++) {
     GstSourceSelect *select = &group->selector[i];
 
     if (select->selector) {
@@ -1181,6 +1665,9 @@ activate_group (GstPlayBin * playbin, GstSourceGroup * group)
   if (!uridecodebin)
     goto no_decodebin;
 
+  /* configure connection speed */
+  g_object_set (uridecodebin, "connection-speed", playbin->connection_speed,
+      NULL);
   /* configure uri */
   g_object_set (uridecodebin, "uri", group->uri, NULL);
 
@@ -1231,7 +1718,7 @@ deactivate_group (GstPlayBin * playbin, GstSourceGroup * group)
 
   GST_DEBUG_OBJECT (playbin, "unlinking group %p", group);
 
-  for (i = 0; i < 3; i++) {
+  for (i = 0; i < GST_PLAY_SINK_TYPE_LAST; i++) {
     GstSourceSelect *select = &group->selector[i];
 
     if (!select->selector)
