@@ -28,7 +28,6 @@
 G_BEGIN_DECLS
 
 #include "gsttcp.h"
-#include "gstfdset.h"
 
 #define GST_TYPE_MULTI_FD_SINK \
   (gst_multi_fd_sink_get_type())
@@ -50,7 +49,7 @@ typedef struct _GstMultiFdSinkClass GstMultiFdSinkClass;
 typedef enum {
   GST_MULTI_FD_SINK_OPEN             = (GST_ELEMENT_FLAG_LAST << 0),
 
-  GST_MULTI_FD_SINK_FLAG_LAST        = (GST_ELEMENT_FLAG_LAST << 2),
+  GST_MULTI_FD_SINK_FLAG_LAST        = (GST_ELEMENT_FLAG_LAST << 2)
 } GstMultiFdSinkFlags;
 
 /**
@@ -68,7 +67,7 @@ typedef enum
   GST_RECOVER_POLICY_NONE,
   GST_RECOVER_POLICY_RESYNC_LATEST,
   GST_RECOVER_POLICY_RESYNC_SOFT_LIMIT,
-  GST_RECOVER_POLICY_RESYNC_KEYFRAME,
+  GST_RECOVER_POLICY_RESYNC_KEYFRAME
 } GstRecoverPolicy;
 
 /**
@@ -93,7 +92,7 @@ typedef enum
   GST_SYNC_METHOD_LATEST_KEYFRAME,
   GST_SYNC_METHOD_BURST,
   GST_SYNC_METHOD_BURST_KEYFRAME,
-  GST_SYNC_METHOD_BURST_WITH_KEYFRAME,
+  GST_SYNC_METHOD_BURST_WITH_KEYFRAME
 } GstSyncMethod;
 
 /**
@@ -110,7 +109,7 @@ typedef enum
   GST_UNIT_TYPE_UNDEFINED,
   GST_UNIT_TYPE_BUFFERS,
   GST_UNIT_TYPE_TIME,
-  GST_UNIT_TYPE_BYTES,
+  GST_UNIT_TYPE_BYTES
 } GstUnitType;
 
 /**
@@ -121,6 +120,7 @@ typedef enum
  * @GST_CLIENT_STATUS_SLOW     : client is too slow
  * @GST_CLIENT_STATUS_ERROR    : client is in error
  * @GST_CLIENT_STATUS_DUPLICATE: same client added twice
+ * @GST_CLIENT_STATUS_FLUSHING : client is flushing out the remaining buffers.
  *
  * This specifies the reason why a client was removed from
  * multifdsink and is received in the "client-removed" signal.
@@ -133,14 +133,17 @@ typedef enum
   GST_CLIENT_STATUS_SLOW        = 3,
   GST_CLIENT_STATUS_ERROR       = 4,
   GST_CLIENT_STATUS_DUPLICATE   = 5,
+  GST_CLIENT_STATUS_FLUSHING    = 6
 } GstClientStatus;
 
 /* structure for a client
  */
 typedef struct {
-  GstFD fd;
+  GstPollFD fd;
 
   gint bufpos;                  /* position of this client in the global queue */
+  gint flushcount;              /* the remaining number of buffers to flush out or -1 if the 
+                                   client is not flushing. */
 
   GstClientStatus status;
   gboolean is_socket;
@@ -197,10 +200,8 @@ struct _GstMultiFdSink {
   GHashTable *fd_hash;  /* index on fd to client */
   guint clients_cookie; /* Cookie to detect changes to the clients list */
 
-  GstFDSetMode mode;
-  GstFDSet *fdset;
-
-  GstFD control_sock[2];/* sockets for controlling the select call */
+  gint mode;
+  GstPoll *fdset;
 
   GSList *streamheader; /* GSList of GstBuffers to use as streamheader */
   gboolean previous_buffer_in_caps;
@@ -249,12 +250,13 @@ struct _GstMultiFdSinkClass {
 		                 GstUnitType format, guint64 value, 
 				 GstUnitType max_unit, guint64 max_value);
   void          (*remove)       (GstMultiFdSink *sink, int fd);
+  void          (*remove_flush) (GstMultiFdSink *sink, int fd);
   void          (*clear)        (GstMultiFdSink *sink);
   GValueArray*  (*get_stats)    (GstMultiFdSink *sink, int fd);
 
   /* vtable */
   gboolean (*init)   (GstMultiFdSink *sink);
-  gboolean (*wait)   (GstMultiFdSink *sink, GstFDSet *set);
+  gboolean (*wait)   (GstMultiFdSink *sink, GstPoll *set);
   gboolean (*close)  (GstMultiFdSink *sink);
   void (*removed) (GstMultiFdSink *sink, int fd);
 
@@ -266,14 +268,14 @@ struct _GstMultiFdSinkClass {
 
 GType gst_multi_fd_sink_get_type (void);
 
-void gst_multi_fd_sink_add (GstMultiFdSink *sink, int fd);
-void gst_multi_fd_sink_add_full (GstMultiFdSink *sink, int fd, GstSyncMethod sync, 
-		GstUnitType min_unit, guint64 min_value,
-		GstUnitType max_unit, guint64 max_value);
-void gst_multi_fd_sink_remove (GstMultiFdSink *sink, int fd);
-void gst_multi_fd_sink_clear (GstMultiFdSink *sink);
-GValueArray* gst_multi_fd_sink_get_stats (GstMultiFdSink *sink, int fd);
-
+void          gst_multi_fd_sink_add          (GstMultiFdSink *sink, int fd);
+void          gst_multi_fd_sink_add_full     (GstMultiFdSink *sink, int fd, GstSyncMethod sync, 
+                                              GstUnitType min_unit, guint64 min_value,
+                                              GstUnitType max_unit, guint64 max_value);
+void          gst_multi_fd_sink_remove       (GstMultiFdSink *sink, int fd);
+void          gst_multi_fd_sink_remove_flush (GstMultiFdSink *sink, int fd);
+void          gst_multi_fd_sink_clear        (GstMultiFdSink *sink);
+GValueArray*  gst_multi_fd_sink_get_stats    (GstMultiFdSink *sink, int fd);
 
 G_END_DECLS
 
